@@ -49,7 +49,7 @@ class Fs_radius_config_class extends Safe_configurable_object_class {
 		$this->set_field_help('authserver','The domain name or IP address of the RADIUS Server.');
 		
 		$this->add_field('auth_port','Authentication Port','string','1812',10);
-		$this->set_field_rules('auth_port','required|greater_than[1]|less_than[65535]');
+		$this->set_field_rules('auth_port','required|valid_port_number');
 		$this->set_field_help('auth_port','The authentication and authorization port of the RADIUS Server.');
 		
 		$this->add_field('shared-secret','RADIUS Shared Secret','string','testing123',60);
@@ -79,7 +79,7 @@ class Fs_radius_config_class extends Safe_configurable_object_class {
 		$this->set_field_help('acctserver','The domain name or IP address of the RADIUS Accounting Server.');*/
 		
 		$this->add_field('acct-port','RADIUS Accounting Port','string','1813',10);
-		$this->set_field_rules('acct-port','required[accounting]|greater_than[1]|less_than[65535]');
+		$this->set_field_rules('acct-port','required[enable-accounting]|valid_port_number');
 		$this->set_field_help('acct-port','The port being used for RADIUS Server Accounting.');
 		
 		$this->create_group('accounting', array('acct-port'));
