@@ -26,6 +26,131 @@ safe_require_class('hardware');
 safe_module_require_class('nsc', 'application');
 safe_module_require_class('fs', 'sip_profile');
 safe_module_require_class('fs', 'sip_request_limit_config');
+class Nsc_sip_domain_profile_param_class extends Safe_configurable_object_class
+{
+    public $_domain_profiles;
+    public $_the_product;
+    public $_the_app;
+    public $_parent_name = null;
+    public $_parent_sip_profile = null;
+    public function __construct($name = null, $parent_sip_profile = null)
+    {
+        if($name != null){
+            if($parent_sip_profile != null){
+                $this->_parent_name = $parent_sip_profile->name();
+                $this->_parent_sip_profile = $parent_sip_profile;
+            }
+            parent::__construct('',$name);
+        }
+    }
+    public function configure($profile)
+    {
+        $this->_the_product = Safe_get_product();
+        $this->_the_app = $this->_the_product->local_node()->software()->application();
+        $this->_domain_profiles = $this->_the_app->domain_profiles(); //Holds all Domain Profiles
+        if($this->_parent_name == null){
+            $this->_parent_name = $profile;
+            $this->_sip_profiles = $this->_the_app->sip_profiles(); //Holds all SIP Profiles
+            $sip_profile = $this->_sip_profiles[$profile];
+            $this->_parent_sip_profile = $sip_profile;
+        }else{
+            $sip_profile = $this->_parent_sip_profile;
+        }
+        $def_name = array();
+        $names = array();
+        $config = $sip_profile->get_data_values();
+        //Set sip profiles
+        if (isset($this->_domain_profiles)) {
+            foreach ($this->_domain_profiles as $profile) {
+                if (isset($profile)) {
+                    $names[$profile->name()] = $profile->name();
+                }
+            }
+        }
+        foreach ($config['domains'] as $sip_domain) {
+            foreach ($this->_domain_profiles as $profile) {
+                if (isset($profile)) {
+                    if (trim(strtolower($profile->name())) == trim(strtolower($sip_domain))) $def_name[$profile->name()] = $profile->name();
+                }
+            }
+        }
+        $this->add_multiple_field('domains', '', 'checkbox', $def_name, $names);
+        $this->set_field_rules('domains', 'callback_check_domains['. implode(',',$names) .']');
+    }
+    
+    public function get_data_value($field){
+        if($field == 'name'){
+            return $this->name();
+        }else{
+            return null;
+        }        
+    }
+    public function set_data_values($data){
+        if($this->_parent_sip_profile){
+            $temp = $this->_parent_sip_profile->get_data_values();
+            parent::set_data_values($data);
+            if (isset($data['domains'])) {
+                foreach ($data['domains'] as $domain) {
+                    if(in_array(trim($domain),$temp['domains'])){
+                        continue;
+                    }else{
+                        $temp['domains'][] = trim($domain);
+                    }
+                }
+            }
+            $this->_parent_sip_profile->set_data_values($temp);
+            return true;
+        }else{
+            return false;
+        }
+    }
+     public function synch(){
+         return;
+     }
+    public function save(){
+        if($this->_parent_sip_profile){
+            return $this->_parent_sip_profile->save();
+        }else{
+            return false;
+        }
+    }
+    public function dispose(){
+        if($this->_parent_sip_profile){
+            $sip = $this->_parent_sip_profile;
+            $profile = $sip->get_data_values();
+            $temp['domains'] = array();
+            for ($a = 0; $a < count($profile['domains']); $a++) {
+                if ($profile['domains'][$a] != $this->name()) $temp['domains'][] = $profile['domains'][$a];
+            }
+            $sip->set_data_values($temp);
+            return $sip->save();
+        }else{
+            return false;
+        } 
+    }
+    public function summary($type = 'horizontal' , $long = false){
+        $table_line = array("Domain" => $this->name());
+        return array('data' => $table_line);
+    }
+    public function validate($data, &$output){
+        unset($data['profile-name']);
+        unset($_POST['profile-name']);
+        return parent::validate($data, $output);
+    }
+    public function check_domains($field_name, $domain_list = null){
+        if($domain_list &&  0 != strlen($domain_list)){
+            $domains = explode(',',$domain_list);
+            $new_domains = parent::get_data_value('domains',false);
+            foreach($new_domains as $new_domain){
+                if(trim($new_domain)!=''  && !in_array(trim($new_domain),$domains))
+                {
+                    return array('check_domains' => 'Unable to bind the domain: '.$new_domain);
+                }
+            }
+            return true;
+        }
+    }
+}
 class Nsc_sip_profile_class extends Fs_sip_profile_class
 {
     private $_media_profiles = array();
@@ -47,7 +172,8 @@ class Nsc_sip_profile_class extends Fs_sip_profile_class
     public function __construct($node, $parent_name, $name)
     {
         // Parent constructor to invoke unserialize if needed
-        parent::__construct($node, $parent_name, $name, $node);
+        parent::__construct($node, $parent_name, $name);
+
         $this->_the_app = $this->_node->software()->application();
     }
     /**
@@ -156,29 +282,13 @@ class Nsc_sip_profile_class extends Fs_sip_profile_class
     protected function configure_session_routing()
     {
         $dialplans = array();
-        $dialplans_msg = array();
-        $dialplans['sip-core-null-routing'] = '-- None --';
-        $dialplans_msg['default'] = '-- None --';
-        //Add dialplans drop downs
-        foreach ($this->_dialplans() as $dialplan) {
-            $real_name = 'user-' . $dialplan->name();
-            $dialplans[$real_name] = $dialplan->name();
-            $dialplans_msg[$real_name] = $dialplan->name();
-        }
-        $this->add_enum_field('call-routing', 'Routing Plan', 'dropdown', 'sip-core-null-routing', $dialplans);
+        $module_routing = $this->_node->software()->application()->find_module_by_name('routing');
+        $dialplans = $module_routing->object_enum('dialplans', ADD_NONE_IF_EMPTY, false);
+        $this->add_enum_field('call-routing', 'Routing Plan', 'dropdown', '', $dialplans);
+        $this->set_field_rules('call-routing', 'required[,in,__selectone__]');
+                
         $this->set_field_help('call-routing', 'XML routing plan for all incoming calls received in this SIP profile.');
-        $this->add_enum_field('sip-dialplan-handler', 'SIP Message Handler', 'dropdown', 'default', $dialplans_msg);
-        /*$this->set_field_help('sip-dialplan-handler','Advanced XML script handler for every SIP message received in this SIP profile during a call.'.
-        safe_form_with_submit_button('You can create a call routing plan in the ',
-        '/SAFe/fs_dialplan/add', 'Call Routing Section', ''));*/
-        $this->set_field_help('sip-dialplan-handler', array(
-            'Advanced XML script handler for every SIP message received in this SIP profile during a call.',
-            '',
-            '<font color="red">Warning: Note this may be very intensive if done for every single call</font>',
-            'An alternative is only setting the channel variable "sip_dialplan_handler" for calls where you need this',
-            '',
-            'You can create routing plan in the <a href="/SAFe/fs_dialplan">Call Routing Section</a>.',
-        ));
+        
         $this->add_enum_field('manual-redirect', 'Manual Redirect Routing', 'dropdown', 'false', $this->enable_disable_enum());
         $this->set_field_help('manual-redirect',
             array('If enabled, all SIP 30X responses (Redirect Responses) will cause a new routing request to your dialplan.',
@@ -192,12 +302,21 @@ class Nsc_sip_profile_class extends Fs_sip_profile_class
 
         // Add Session routingcategory
         $this->set_field_category('call-routing', 'Session Routing');
+        
+        //hmr drop downs
+        $module_hmr = $this->_node->software()->application()->find_module_by_name('hmr');
+        $hmrs = $module_hmr->object_enum('dialplans', ADD_NONE_ALWAYS, true);
+        $this->add_enum_field('ingress', 'Ingress ', 'dropdown', '__none__', $hmrs);
+        $this->add_enum_field('egress', 'Egress  ', 'dropdown', '__none__', $hmrs);
+        
+        // Add Session routingcategory
+        $this->set_field_category('ingress', 'Header Manipulation');
     }
     protected function configure_nat()
     {
         // Eventually we'll allow the user to add their own ACLs
         $acl_list = array();
-        $acl_list[''] = '-- None --';
+        $acl_list[''] = '(None)';
         $acl_list['nat.auto'] = 'RFC1918';
         $this->add_enum_field('apply-nat-acl', 'NAT ACL', 'dropdown', '', $acl_list);
         $this->set_field_help('apply-nat-acl', 'Apply a given network ACL to the host IP of the Contact: header in SIP requests to determine if a host is behind NAT');
@@ -287,6 +406,7 @@ class Nsc_sip_profile_class extends Fs_sip_profile_class
         // Add NAT category
         $this->set_field_category('apply-nat-acl', 'NAT Traversal');
     }
+    
     public function configure()
     {
         parent::configure();
@@ -298,7 +418,80 @@ class Nsc_sip_profile_class extends Fs_sip_profile_class
         $this->configure_load_limits();
         $this->configure_session_routing();
         //Adds hidden record in db to hold domains
-        $this->add_field('domains', 'domains', 'hidden', array());
+        $this->add_field('domains', 'domains', 'private', array());
+        
+        // Register objects
+        $this->register_aggregate_object('domain',
+                array(
+                        'name' => 'Domain',
+                        'base_path' => false, // need to verify path
+                        'dynamic' => true,
+                        'pseudo'  => true,  
+                        /* Add some extra infos for config_manager create object */
+                        'controller' => array(
+                                'config_manager' => array(
+                                        'summary_buttons' => 'Delete[Unbind]:confirm',
+                                        'allow_create_button' => 'Add[Bind]'
+                                ),
+                        ),
+                        'autoname' => true,
+                        'popup_add_form' => true,
+                        'methods' => array(
+                                'create' => array(
+                                        'name' => 'Create',
+                                        'description' => 'Create a domain',
+                                        'request' => 'POST',
+                                ),
+                                'retrieve' => array(
+                                        'name' => 'Retrieve',
+                                        'description' => 'Retrieve a domain',
+                                        'request' => 'GET',
+                                ),
+                                'update' => array(
+                                        'name' => 'Update',
+                                        'description' => 'Update a domain',
+                                        'request' => 'POST',
+                                ),
+                                'delete' => array(
+                                        'name' => 'Delete',
+                                        'description' => 'Delete a domain',
+                                        'request' => 'POST',
+                                ),
+                        ),
+                )
+        );
+       
+        
+        $this->register_aggregate_object('limit',
+                array(
+                        'name' => 'Limits',
+                        'base_path' => $this->object_name() . '/limits', // need to verify path
+                        'dynamic' => true,
+                        'autoname' => true,
+                        'methods' => array(
+                                'create' => array(
+                                        'name' => 'Create',
+                                        'description' => 'Create a limit',
+                                        'request' => 'POST',
+                                ),
+                                'retrieve' => array(
+                                        'name' => 'Retrieve',
+                                        'description' => 'Retrieve a limit',
+                                        'request' => 'GET',
+                                ),
+                                'update' => array(
+                                        'name' => 'Update',
+                                        'description' => 'Update a limit',
+                                        'request' => 'POST',
+                                ),
+                                'delete' => array(
+                                        'name' => 'Delete',
+                                        'description' => 'Delete a limit',
+                                        'request' => 'POST',
+                                )
+                        )
+                )
+        );
     }
     /**
      * Pulls all media profiles to populate form
@@ -317,6 +510,12 @@ class Nsc_sip_profile_class extends Fs_sip_profile_class
         $this->_dialplans = $this->_the_app->dialplans();
         return $this->_dialplans;
     }
+    private function _hmrs()
+    {
+        $module = $this->_node->software()->application()->find_module_by_name('hmr');
+        $this->_hmrs = $module->dialplans();
+        return $this->_hmrs;
+    }
     private function _domains()
     {
         $this->_domains = $this->_the_app->domain_profiles();
@@ -372,6 +571,20 @@ class Nsc_sip_profile_class extends Fs_sip_profile_class
         }
         return false;
     }
+    public function validate($data, &$output){
+        $domains = array_keys($this->_domains());
+        if(isset($data['domains']) && is_array($data['domains']) ){
+            foreach($data['domains'] as $item){
+                if(in_array($item, $domains)){
+                    continue;
+                }else{
+                    $output = 'Unknown domain : ' .$item ;
+                    return false;
+                }
+            }                        
+        }
+        return parent::validate($data, $output);
+    }
     /**
      * Check to see if the sip profile can be deleted
      * @see Safe_serializable_object_class::can_dispose()
@@ -380,41 +593,190 @@ class Nsc_sip_profile_class extends Fs_sip_profile_class
      */
     public function can_dispose(&$reason)
     {
-        $can_dispose = TRUE;
-        // Check trunks
-        /* TODO(wadam) - Skip trunk validation for now
-        $gateways = $this->_gateways();
-        foreach($gateways as $gateway){
-        $prof_data = $gateway->get_data_values(false);
-        if ($prof_data['sip_profile'] == $this->name()) {
-        $can_dispose = FALSE;
-        $reason[] = 'SIP Trunk = ' . $gateway->name();
+        $usages = $this->node()->software()->application()->check_object_usage('sip','profile', $this->name());
+        $path = $this->object_name();
+        $module = $this->node()->software()->application()->find_module_by_path($path);
+        $usages = array_merge($usages, $module->check_object_usage('sip', 'profile', $this->name()));
+        if($usages){
+            $reason['msg'] = "  {$this->name()} is used by ";
+            $reason['obj'] = $usages;
+            return false;
         }
+        return parent::can_dispose($reason);
+    }
+    
+    public function limits()
+    {
+        return $this->get_aggregate_objects('limit');
+    }
+    /**
+     * @brief
+     *
+     * @param[in out] $name
+     * @param[in out] $data
+     *
+     * @return
+     */
+    public function api_create_limit($name, $data=null) {
+        $condition = new Fs_sip_request_limit_config_class($this->node(), $this->object_name() . '/limits', $name);
+        $condition->configure();
+        if($data) {
+            $condition->set_data_values($data);
         }
-        */
-        // Check CAC rules
-        $cac_rules = $this->_cac_rules();
-        foreach ($cac_rules as $cacs) {
-            $prof_data = $cacs->get_data_values(false);
-            if (sizeof($prof_data['sip-profiles']) != 0) {
-                foreach ($prof_data['sip-profiles'] as $sip) {
-                    if ($sip == $this->name()) {
-                        $can_dispose = FALSE;
-                        $reason[] = 'Call Admission = ' . $cacs->name();
-                    }
+        return $condition;
+    }
+    /**
+     * @brief
+     *
+     * @param[in out] $name
+     * @param[in out] $data
+     *
+     * @return
+     */
+    public function api_retrieve_limit($name, $data=null) {
+        $limits = $this->limits();
+        return $limits[$name];
+    }
+    /**
+     * @brief
+     *
+     * @param[in out] $name
+     * @param[in out] $data
+     *
+     * @return
+     */
+    public function api_update_limit($name, $data=null, &$output = null) {
+        $limit = $this->api_retrieve_limit($name);
+        if($limit) {
+            if ($limit->validate($data,$output)) {
+                if (true == $limit->save()) {
+                    return true;
                 }
             }
         }
-        // Check if this sip profile is used by a domain
-        $domains = $this->_domains();
-        foreach ($domains as $domain) {
-            if ($domain->is_sip_profile_used($this->name())) {
-                $can_dispose = FALSE;
-                $reason[] = 'Domain = ' . $domain->name();
+        return false;
+    }
+    /**
+     * @brief
+     *
+     * @param[in out] $name
+     * @param[in out] $data
+     *
+     * @return
+     */
+    public function api_delete_limit($name, $data=null, &$output = null) {
+        $limits = $this->limits();
+        if($limits[$name]) {
+            if($limits[$name]->can_dispose($output)) {
+                return $limits[$name]->dispose();
             }
         }
-        if (!$can_dispose) return $can_dispose;
-        return parent::can_dispose();
+        return false;
+    }
+    public function domains()
+    {
+        $domains = $this->get_data_value('domains');
+        $domain_objs = array();
+        if($domains){
+            foreach($domains as $domain_item){
+                $domain_objs[$domain_item] = new Nsc_sip_domain_profile_param_class($domain_item, $this);
+                $domain_objs[$domain_item]->configure($this->name());
+            }
+        }
+        return $domain_objs;
+    }
+    /**
+     * @brief
+     *
+     * @param[in out] $name
+     * @param[in out] $data
+     *
+     * @return
+     */
+    public function api_create_domain($name, $data=null) {
+        $domain = new Nsc_sip_domain_profile_param_class($name, $this);
+        $domain->configure($this->name());
+        if($data) {
+            $domain->set_data_values($data);
+        }
+        return $domain;
+    }
+    /**
+     * @brief
+     *
+     * @param[in out] $name
+     * @param[in out] $data
+     *
+     * @return
+     */
+    public function api_retrieve_domain($name, $data=null) {
+        $domains = $this->domains();
+        if($name){
+            return $domains[$name];
+        }else{
+            return $domains;
+        }
+        
+    }
+    /**
+     * @brief
+     *
+     * @param[in out] $name
+     * @param[in out] $data
+     *
+     * @return
+     */
+    public function api_delete_domain($name, $data=null, &$output = null) {
+        $domains = $this->domains();
+        if($domains[$name]) {
+            if($domains[$name]->can_dispose($output)) {
+                return $domains[$name]->dispose();
+            }
+        }
+        return false;
+    }
+    //re define the add form for popup dialog
+    public function new_fields(&$config_manager, $type)
+    {
+        if($type == 'domain'){
+            $name = Safe_object_serializer_class::build_unique_name($type);
+            //add new domain
+            $config_manager->add_field('profile-name', 'Profile Name', 'hidden', $name, 50);
+            $domain_obj = new Nsc_sip_domain_profile_param_class($name, $this);
+            $domain_obj->configure($this->name());
+            $domain_cfg = $domain_obj->get_data();
+            $def_name = array();
+            $value_names = array();
+            if(is_array($domain_cfg['domains']['default'])){
+                $def_name = array();
+                foreach($domain_cfg['domains']['default'] as $tmp_def){
+                    $def_name[$tmp_def] = $tmp_def;
+                }
+            }
+            if(is_array($domain_cfg['domains']['value']) && $domain_cfg['domains']['value'] != null){
+                $value_names = array();
+                foreach($domain_cfg['domains']['value'] as $tmp_val){
+                    $value_names[$tmp_val] = $tmp_val;
+                }
+            }else{
+                $config_manager->error = true;
+                $config_manager->error_message = "No domain defined!";
+            }
+            $config_manager->add_multiple_field('domains', '', $domain_cfg['domains']['type'],$def_name ,$value_names);
+            $config_manager->set_field_rules('domains','callback_check_domain');
+            //pass by the edit form, save directly
+            $config_manager->add_field('save', 'Save', 'hidden', 'save', 10);
+        }else{
+            parent::new_fields($config_manager, $type);
+        }
+    }
+    public function check_domain($field ,$param, $postdata)
+    {
+        if(!is_array($postdata) || $postdata == null){
+            $error['check_domain'] = ' No domain selected.';
+            return $error;
+        }
+        return true;
     }
 }
 ?>
