@@ -38,13 +38,10 @@ class Safe_application_class extends Safe_service_class
     {
         return 'Application';
     }
-    public function allow_rest(){
-        return true;
-    }
     public function configure()
     {
         // Preferences
-        $this->_preferences = new Safe_application_settings_class($this->node()->software());
+        $this->_preferences = new Safe_application_settings_class($this->path() . '/application');
         $this->_preferences->configure();
         $this->_preferences->synch();
         $license = $this->node()->software()->license();
@@ -66,6 +63,7 @@ class Safe_application_class extends Safe_service_class
         $this->register_aggregate_object( 'configuration',
             array(
                 'name' => 'Configuration',
+                'pseudo' => true,
                 'description' => array(
                     'Manages overall Application configuration.',
                     ),
@@ -95,6 +93,7 @@ class Safe_application_class extends Safe_service_class
             array(
                 'name' => 'License',
                 'singleton' => true,
+                'base_path' => false,
                 'description' => array(
                     'Manages Application licensed Features.',
                     ),
@@ -122,6 +121,7 @@ class Safe_application_class extends Safe_service_class
                     'Manages Application components version.',
                     ),
                 'singleton' => true,
+                'base_path' => false,
                 'methods' => array(
                     'retrieve' => array(
                         'name' => 'Retrieve',
@@ -138,16 +138,17 @@ class Safe_application_class extends Safe_service_class
                     'Manages Application preferences.',
                     ),
                 'singleton' => true,
+                'base_path' => false,
                 'methods' => array(
                     'retrieve' => array(
                         'name' => 'Retrieve',
                         'description' => array( 'Returns Application preferences.' ),
-                        'request' => '',
+                        'request' => 'GET',
                     ),
                     'update' => array(
                         'name' => 'Update',
                         'description' => array( 'Modifies Application preferences.' ),
-                        'request' => '',
+                        'request' => 'POST',
                     ),
                 ),
             )
@@ -183,8 +184,66 @@ class Safe_application_class extends Safe_service_class
             }
         }
         return parent::find_module_by_name($name);
-
     }
+    /**
+     * @brief find obj by module name, type, obj name (either in modules or
+     * services)
+     *
+     * @param
+     *            [in out] $name
+     *            
+     * @return
+     *
+     */
+    public function find_obj_by_name($module_name, $type, $obj_name, $sub_type = null, $sub_obj_name = null) {
+        $module = $this->find_module_by_name ( $module_name );
+        if($module){
+            $obj_types = $module->has_aggregate_objects ();
+            if ( isset ( $obj_types [$type] ) && $obj_types [$type]['configurable']) {
+                $objs = $module->get_aggregate_objects ( $type );
+                if (isset ( $objs [$obj_name] )) {
+                    if ($sub_type != null && $sub_obj_name != null) {
+                        $tmp_obj = $objs [$obj_name];
+                        $sub_obj_types = $tmp_obj->has_aggregate_objects ();
+                        if ( isset ( $sub_obj_types[$sub_type] ) && $sub_obj_types[$sub_type]['configurable']) {
+                            $sub_objs = $tmp_obj->get_aggregate_objects ( $sub_type );
+                            if (isset ( $sub_objs [$sub_obj_name] )) {
+                                return $sub_objs [$sub_obj_name];
+                            } else {
+                                return false;
+                            }
+                        }
+                    } else {
+                        return $objs [$obj_name];
+                    }
+                } 
+            }
+        }
+        return false;
+    }
+    public function find_obj_by_path($path) {
+        $path =  str_replace('/NSC/local/software/', '', $path);
+        $path =  str_replace('/NSC/local/hardware/', '', $path);
+        $module = null;
+        $type = null;
+        $name = null;
+        $sub_type = null;
+        $sub_name = null;
+        list ( $module, $type, $name, $sub_type, $sub_name ) = explode ( '/', $path );
+        return $this->find_obj_by_name ( $module, $type, $name, $sub_type, $sub_name );
+    }
+    public function find_module_by_path($path) {
+        $path =  str_replace('/NSC/local/software/', '', $path);
+        $path =  str_replace('/NSC/local/hardware/', '', $path);
+        $module = null;
+        $type = null;
+        $name = null;
+        $sub_type = null;
+        $sub_name = null;
+        list ( $module, $type, $name, $sub_type, $sub_name ) = explode ( '/', $path );
+        return $this->find_module_by_name ( $module );
+    }
+    
     /**
      * @brief
      *         
@@ -201,12 +260,25 @@ class Safe_application_class extends Safe_service_class
                 'pattern' => $cfg_manager->directory('log') . '/' . $this->name() . '.log*',
                 'global_capabilities' => array(
                     'download',
-                    'rotate'
+                    //'rotate'
                 ) ,
                 'capabilities' => array(
                     'download',
+                    'delete',
                     'view'
-                )
+                ),
+                'methods' => array(
+                        'download' => array(
+                                'name' => 'Download',
+                                'description' => array( 'Download log files.' ),
+                                'request' => 'GET',
+                        ),
+                        'delete' => array(
+                                'name' => 'Delete',
+                                'description' => array( 'Delete a  Core Dump files.' ),
+                                'request' => 'POST',
+                        ),
+                ),
             ) ,
             'core' => array(
                 'description' => 'Core Dump',
@@ -218,7 +290,19 @@ class Safe_application_class extends Safe_service_class
                 'capabilities' => array(
                     'download',
                     'delete'
-                )
+                ),
+                'methods' => array(
+                        'download' => array(
+                                'name' => 'Download',
+                                'description' => array( 'Download a Core Dump files.' ),
+                                'request' => 'GET',
+                        ),
+                        'delete' => array(
+                                'name' => 'Delete',
+                                'description' => array( 'Delete a  Core Dump files.' ),
+                                'request' => 'POST',
+                        ),
+                ),
             ) ,
             'corebt' => array(
                 'description' => 'Core Dump Back Trace',
@@ -232,7 +316,19 @@ class Safe_application_class extends Safe_service_class
                     'download',
                     'delete',
                     'view'
-                )
+                ),
+                'methods' => array(
+                        'download' => array(
+                                'name' => 'Download',
+                                'description' => array( 'Download a Core Dump files.' ),
+                                'request' => 'GET',
+                        ),
+                        'delete' => array(
+                                'name' => 'Delete',
+                                'description' => array( 'Delete a  Core Dump files.' ),
+                                'request' => 'POST',
+                        ),
+                ),
             ) ,
         );
         // Get license info
@@ -245,15 +341,24 @@ class Safe_application_class extends Safe_service_class
                         'filesystem' => true,
                         'singleton' => true,
                         'pattern' =>  $cfg_manager->directory('base') . '/upload/license.tgz',
+                        'file_ext' => 'tgz|gz|tar.gz',
                         'global_capabilities_no_data' => array(
                                 'upload'
+                        ) ,
+                        'methods' => array(
+                            'upload' => array(
+                                    'name' => 'Upload',
+                                    'description' => array( 'Upload License.' ),
+                                    'request' => 'POST',
+                            )
                         )
                 );
         }
         if (isset($archive_manager)) $data['archive'] = array(
-            'description' => 'Backup',
-            'filesystem' => TRUE,
-            'pattern' => $archive_manager->directory(TRUE) ,
+            'description' => 'Archive',
+            'filesystem' => true,
+            'pattern' => $archive_manager->directory(true) ,
+            'file_ext' => 'tgz|gz|tar.gz',
             'global_capabilities_no_data' => array(
                 'backup',
                 'upload'
@@ -262,7 +367,34 @@ class Safe_application_class extends Safe_service_class
                 'download',
                 'delete',
                 'restore'
-            )
+            ),
+            'methods' => array(
+                    'backup' => array(
+                            'name' => 'Backup',
+                            'description' => array( 'Backup system settings.' ),
+                            'request' => 'POST',
+                    ),
+                    'download' => array(
+                            'name' => 'Download',
+                            'description' => array( 'Download a backup files.' ),
+                            'request' => 'GET',
+                    ),
+                    'upload' => array(
+                            'name' => 'Upload',
+                            'description' => array( 'Upload a backup files.' ),
+                            'request' => 'POST',
+                    ),
+                    'delete' => array(
+                            'name' => 'Delete',
+                            'description' => array( 'Delete a backup files.' ),
+                            'request' => 'POST',
+                    ),
+                    'restore' => array(
+                            'name' => 'Restore',
+                            'description' => array( 'Restore a backup files.' ),
+                            'request' => 'POST',
+                    )
+            ),
         );
         return $data;
     }
@@ -291,6 +423,22 @@ class Safe_application_class extends Safe_service_class
         }
         return parent::get_data_files($data_name);
     }
+    public function data_summary($data_name){
+        if('archive' == $data_name){
+            $the_product = Safe_get_product();
+            $cfg_mgr = $the_product->local_node()->configuration_manager();
+            $archive_mgr = $cfg_mgr->archive_manager();
+            $data_files = $this->get_data_files($data_name);
+            $data_summary = array();
+            foreach ($data_files as $key => $file) {
+                $info = $archive_mgr->info($key);
+                $data_summary[] = array('Name' => $key , 'Type' => $info['type']);
+            }
+            return $data_summary;
+        }else{
+            return parent::data_summary($data_name);
+        }
+    }
     /**
      * @brief Return data description
      * Special hook for license
@@ -364,6 +512,55 @@ class Safe_application_class extends Safe_service_class
         }
         return $conflicts;
     }
+    public function check_object_usage($src_module_name, $obj_type, $obj_name, $sub_typ=null, $sub_name=null)
+    {
+        // Parent function
+        $usages = parent::check_object_usage($src_module_name, $obj_type, $obj_name, $sub_typ, $sub_name) ;
+        // Take care of services
+        foreach($this->node()->software()->services() as $service) {
+            if($service->name() != $this->name() && $src_module_name != $service->name()) {
+                $sub_usages = array();
+                $sub_usages = $service->check_object_usage($src_module_name, $obj_type, $obj_name, $sub_typ, $sub_name) ;
+                if($sub_usages){
+                    $usages = array_merge($usages, $sub_usages);
+                }
+            }
+        }
+        return $usages;
+    }
+    public function check_object_modified($src_module_name, $obj_type, $obj_name, $sub_typ=null, $sub_name=null)
+    {
+        // Parent function
+        $modifies = parent::check_object_modified($src_module_name, $obj_type, $obj_name, $sub_typ, $sub_name) ;
+        // Take care of services
+        foreach($this->node()->software()->services() as $service) {
+            if($service->name() != $this->name() && $src_module_name != $service->name()) {
+                $sub_modifies = array();
+                $sub_modifies = $service->check_object_usage($src_module_name, $obj_type, $obj_name, $sub_typ, $sub_name) ;
+                if($sub_modifies){
+                    $modifies = array_merge($modifies, $sub_modifies);
+                }
+            }
+        }
+        return $modifies;
+    }
+    /**
+     * @brief Check if update package can be install
+     *
+     * @param[in out] $reason
+     *
+     * @return true/false
+     */
+    public function can_install_data($info, &$reason)
+    {
+        if('update' == $info['type']){
+            if (Safe_service_class::STATUS_RUNNING == $this->status()) {
+                $reason[] = $this->description() . ' is Running';
+                return false;
+            }
+        }
+        return parent::can_install_data($info, $reason);
+    }
     /**
      * @brief
      *
@@ -448,6 +645,13 @@ class Safe_application_class extends Safe_service_class
         }else{
             $result['modified'] = false; 
         }
+        $checklist = array();
+        $this->node()->checklist($checklist);
+        if($checklist){
+            $result['checklist'] = $checklist;
+        }
+        // Add configuration generate status: count, last generate date
+        $result['generate'] = $this->node()->configuration_manager()->generate_status();
         return $result;
     }
     /**
@@ -479,31 +683,37 @@ class Safe_application_class extends Safe_service_class
                 $output['sys_running'] ='System is Running';
                 $rc = false;
             }else{
-                $this->modified_object_list($reload_list);
+                if ($restart) {
+                    $this->modified_object_list($reload_list, false);
+                }else{
+                    $this->modified_object_list($reload_list);
+                }
                 Safe_enter_critical_section();
                 if (!$this->node()->generate_config()) {
-                    $output['apply_failed'] = 'Apply Configuration Failed';
-                    $output['apply_failed_data'] = $this->node()->configuration_manager()->progress();
+                    $output['message'] = 'Apply configuration failed.';
+                    $output['reason'] = $this->node()->configuration_manager()->progress();
                     $rc = false;
                 } else {
                     if ($restart) {
-                        $rc_restart = $this->restart();
+                        $reason = array();
+                        $rc_restart = $this->restart($force, $reason);
                         if (0 == $rc_restart) {
-                            $output['restart_succeeded'] = 'System Apply Configuration and Restart Succeeded';
+                            $output['restart_succeeded'] = 'System apply configuration and restart succeeded.';
                             $rc = true;
                         } else {
-                            $output['restart_failed'] = 'System Apply Configuration and Restart Failed';
+                            $output['restart_failed'] = 'System apply configuration and restart failed.';
+                            $output['restart_failed_data'] = $reason;
                             $rc = false;
                         }
                     }
-                    // Check if some service with modified config and 
+                    // Check if some service with modified config and
                     // supporting reload and running -> post_write
                     foreach($reload_list as $module_name => $module_obj_list) {
                         $module = $this->find_module_by_name($module_name);
                         // TODO - need to loop around obj_type
                         if(is_subclass_of($module, "Safe_service_class")
-                            &&
-                            $module->status == Safe_service_class::STATUS_RUNNING ) {
+                                &&
+                                $module->status == Safe_service_class::STATUS_RUNNING ) {
                             if(!$module->post_write_config()) {
                                 $output[] = 'Post Write Failed for '.$module->name() . '.';
                             }
@@ -545,22 +755,27 @@ class Safe_application_class extends Safe_service_class
                 $module = $this->find_module_by_name($module_name);
                 // TODO - need to loop around obj_type
                 if(!$module->reload_generate_config($cfg_mgr)) {
-                    $output[] = 'Generate Configuration Failed for ' . $module->name() . '.';
+                    $output[] = 'Generate configuration failed for ' . $module->name() . '.';
                     $rc = false;
                 }
             }    
+            // Ensure application reload_generate is called
+            if(!$this->reload_generate_config($cfg_mgr)) {
+                $output[] = 'Generate configuration failed for ' . $this->name() . '.';
+                $rc = false;
+            }
         }
         if($rc) {
             // Step 2
             if($cfg_mgr->errors()) {
-                $output['cfg_mgr'] = 'Apply Configuration Failed.';
+                $output['cfg_mgr'] = 'Apply configuration failed.';
                 $output['cfg_mgr_data'] = $cfg_mgr->progress();
                 $rc = false;
             }
             if($rc) {
                 // Step 3
                 if(!$cfg_mgr->write_config($this->node())) {
-                    $output[] = 'Write Configuration Failed.';
+                    $output[] = 'Write configuration failed.';
                     $rc = false;
                 }
                 // Step 4
@@ -577,19 +792,35 @@ class Safe_application_class extends Safe_service_class
                     // TODO - need to loop around obj_type
                     $module->reload_clear_modified(true);
                 }
+                // Application post_reload
+                if(!$this->post_reload()) {
+                    $output[] = 'Post Reload Failed for '.$this->name() . '.';
+                }
             }
         }
         Safe_leave_critical_section();
         return $rc;
     }
 
+    
     public function api_create_preferences($name=null, $data = null, &$output = null){
-        // Cannot create one
-        return $this->api_retrieve_preferences();
+        //for documentation list use
+        return $this->_preferences;
     }
     public function api_retrieve_preferences($name=null, $data = null, &$output = null){
         return $this->_preferences;
     }
+    public function api_update_preferences($name=null, $data=null, &$output = null) {
+        $preferences = $this->api_retrieve_preferences($name);
+        if($preferences) {
+            if ($preferences->validate($data,$output)) {
+                if (true == $preferences->save()) {
+                    return true;
+                }
+            }
+        }
+        return false;
+    }
 
     public function api_create_license($name=null, $data = null, &$output = null){
         // Cannot create one
@@ -601,6 +832,16 @@ class Safe_application_class extends Safe_service_class
     public function api_info_license($name=null, $data = null, &$output = null){
         return $this->api_retrieve_system_info();
     }
+    public function api_upload_license($name=null, $data = null, &$output = null){
+        $data_def = $this->get_data_settings();
+        $upload_api_method  = $data_def['license']['methods']['upload']['request'];
+        if($upload_api_method){
+            return $this->upload_data('license', $name , $output , $upload_api_method);
+        }else{
+            $output['message'] = 'Upload request method error.';
+            return false;
+        }
+    }
 
     public function api_create_version($name=null, $data = null, &$output = null){
         // Cannot create one
@@ -641,5 +882,127 @@ class Safe_application_class extends Safe_service_class
 
         return $info;
     }
+    public function api_download_log($name=null, $data = null, &$output = null){
+        $this->download_data('log', $name);
+        exit();
+    }
+    public function api_delete_log($name=null, $data = null, &$output = null){
+        if ($name){
+            return $this->delete_data('log', $name);
+        }else{
+            //not allow download all files
+            return false;
+        }
+    }
+    public function api_download_core($name=null, $data = null, &$output = null){
+        if ($name){
+            $this->download_data('core', $name);
+            exit();
+        }else{
+            //not allow download all files
+            return false;
+        }
+        
+    }
+    public function api_delete_core($name=null, $data = null, &$output = null){
+        return $this->delete_data('core', $name);
+    }
+    public function api_download_corebt($name=null, $data = null, &$output = null){
+        $this->download_data('corebt', $name);
+        exit();
+    }
+    public function api_delete_corebt($name=null, $data = null, &$output = null){
+        return $this->delete_data('corebt', $name);
+    }
+    
+    public function api_backup_archive($name=null, $data = null, &$output = null){
+        $the_product = Safe_get_product();
+        $cfg_mgr = $the_product->local_node()->configuration_manager();
+        $archive_mgr = $cfg_mgr->archive_manager();
+        $reason = array();
+        if ($the_product->can_backup($reason)) {
+            $the_product->pre_backup($name, $reason);
+            // Create archive
+            $archive = $archive_mgr->create_archive($name);
+            // Post backup hook
+            $the_product->post_backup($name, $reason);
+            if (FALSE !== $archive) {
+                return array('name' => $archive);
+            } else {
+                // Display error
+                $output['notif_error'] = 'Fail to create archive';
+                return false;
+            }
+        }else{
+            $output['notif_error'] = $reason;
+            return false;
+        }
+    }
+    public function api_backup_archive_parameters()
+    {
+        $the_product = Safe_get_product();
+        $cfg_mgr = $the_product->local_node()->configuration_manager();
+        $archive_mgr = $cfg_mgr->archive_manager();
+        return $archive_mgr->get_archive_configurable_object();
+    }
+    public function api_download_archive($name=null, $data = null, &$output = null){
+        if ($name){
+            $this->download_data('archive', $name);
+        }else{
+            return false;
+        }
+    
+    }
+    public function api_upload_archive($name=null, $data = null, &$output = null){
+        $data_def = $this->get_data_settings();
+        $upload_api_method  = $data_def['archive']['methods']['upload']['request'];
+        if($upload_api_method){
+            return $this->upload_data('archive', $name , $output , $upload_api_method);
+        }else{
+            $output['message'] = 'Upload request method error.';
+            return false;
+        }
+    }
+    public function api_delete_archive($name=null, $data = null, &$output = null){
+        if ($name){
+            return $this->delete_data('archive', $name);
+        }else{
+            return false;
+        }
+    }
+    public function api_restore_archive($name=null, $data = null, &$output = null){
+        $the_product = Safe_get_product();
+        $cfg_mgr = $the_product->local_node()->configuration_manager();
+        $archive_mgr = $cfg_mgr->archive_manager();
+        $reasons = array();
+        $info = $archive_mgr->info($name);
+        if ($archive_mgr->can_restore($name, $reasons) && $the_product->can_restore($name, $reasons)) {
+            // Loop through all service/modules for pre-restore
+            if (!($rc = $the_product->pre_restore($info, $arch, $reasons))
+                // Restore
+                || !($rc = $archive_mgr->restore_archive($name)) 
+                || !($rc = $the_product->post_restore($info, $reasons))
+                    ) {
+                $output['notif_error'] = $reasons;
+                return false;
+            }
+        }else{
+            $output['notif_error'] = $reasons;
+            return false;
+        }
+        return true;
+    }
+    protected function get_hook_script($step , $type)
+    {
+        $script = parent::get_hook_script($step, $type);
+        if(!$script){
+            $base = $this->node()->configuration_manager()->directory('base');
+            $_script = $base.'/hooks/application.' . $type .'.'.$step;
+            if(file_exists($_script)) {
+                return $_script;
+            }
+        }
+        return $script;
+    }
 }
 /* End of file safe_product_class.php */
