@@ -313,6 +313,18 @@ class Safe_module_class extends Safe_object_class
     {
         return array();
     }
+    /**
+     * @brief Return module action settings
+     *         If module can expose action(s) it should return an array formatted as follow:
+     *         $array= <action_name> (
+     *         ['description'] => action description)
+     *
+     * @return array as describe above
+     */
+    public function get_action_settings()
+    {
+        return array();
+    }
     public function get_aggregate_object_method_params($type, $method)
     {
         $param = parent::get_aggregate_object_method_params($type, $method);
@@ -565,7 +577,6 @@ class Safe_module_class extends Safe_object_class
             $data_cfg = $data_cfg_setting[$data_name];
             if($from == 'PUT'){
                 //convert put upload file to post upload file
-                $_POST['Upload'] = 'YES';
                 if (!isset($_SERVER['HTTP_UPLOAD_FILE_NAME'])) {
                     $output['notif_error'][] = 'Fail to upload ' . $this->description() . ' ' . $data_cfg['description'];
                     return false;
@@ -586,6 +597,8 @@ class Safe_module_class extends Safe_object_class
                     $_FILES['archive']['from'] = 'PUT';
                 }
             }
+            $_POST['Upload'] = 'YES';
+
             $upload_lib = new Safe_upload_class();
             if ($upload_lib->is_uploading()) {
                 if($data_cfg['singleton']){
@@ -608,6 +621,9 @@ class Safe_module_class extends Safe_object_class
                     if($from == 'PUT'){
                         @unlink($tmpfname);
                     }
+                    $output['name'] = $up_data['file_name'];
+                    $output['size'] = (int)ceil(1024 * $up_data['file_size']);
+                    error_log($output['size']);
                     if($data_cfg['singleton']){
                         //after upload ini
                         $output_msg = array();
@@ -1040,5 +1056,51 @@ class Safe_module_class extends Safe_object_class
     {
         return $this->name();
     }
+    /**
+     * @brief Check conflict for a particular resource
+     *  Return an array of object description:
+     *  'module'   => Module description
+     *  'obj_type' => Object type description
+     *  'obj_name' => Object name 
+     *  'sub_type' => (opt) Child type description
+     *  'sub_name' => (opt) Child Object name
+     *  'controller_url' => (opt) controller url set in module 
+     *  'url' =>  Object url for edit
+     *
+     * @param[in out] $obj_data - (array) "obj_type", "obj_name", "sub_type", "sub_name"
+     * 
+     * @return
+     */
+    public function get_object_description($obj_data){
+        $description = array();
+        $url = '';
+        $controller_url = '';
+        $description['module'] = $this->description();
+        if(isset($obj_data['obj_type'])){
+            $obj_type_def = $this->get_aggregate_object_definition($obj_data['obj_type']);
+            $description['obj_type'] = $obj_type_def['name'];
+            if(isset($obj_type_def['controller_url'])){
+                $description['controller_url'] = $obj_type_def['controller_url'];
+            }
+            if(isset($obj_data['obj_name'])){
+                $url = "/{$obj_data['name']}/{$obj_data['obj_type']}/{$obj_data['obj_name']}";
+                $description['obj_name'] = $obj_data['obj_name'];
+                if(isset($obj_data['sub_type']) && isset($obj_data['sub_name'])){
+                    $aggr_obj = $this->get_aggregate_objects($obj_data['obj_type']);
+                    $obj = $aggr_obj[$obj_data['obj_name']];
+                    if($obj){
+                        $sub_obj_type_def = $obj->get_aggregate_object_definition($obj_data['sub_type']);
+                        $description['sub_type'] = $sub_obj_type_def['name'];
+                        $description['sub_name'] = $obj_data['sub_name'];
+                        $url = $url . "/{$obj_data['sub_type']}/{$obj_data['sub_name']}";
+                    }
+                }
+            }
+        }
+        if($url){
+            $description['url'] = $url;
+        }
+        return $description;
+    }
 }
 /* End of file safe_module_class.php */
