@@ -9,20 +9,139 @@
  * @desc allows to disable all fields starting with the end of a field name
  */
 function safe_disabled(g, v) {
-  if (group[g]!=undefined) {
-    var fields = group[g];
+  if (typeof group[g] !== 'undefined' && typeof group[g]['fields'] !== 'undefined') {
+    var fields = group[g]['fields'];
     for(var i=0;i<fields.length;i++) {
-      if (v == 'false') {
-        $("[name='"+fields[i]+"']").attr('disabled','disabled');
-        $("[name='"+fields[i]+"']").addClass('field_disabled');
+       if (v == group[g]['default']['value'] ) {
+        if(group[g]['default']['status']=='disable'){
+        	$("[name='"+fields[i]+"']").attr('disabled','disabled');
+            $("[name='"+fields[i]+"']").addClass('field_disabled');
+        }else{
+        	$("[name='"+fields[i]+"']").removeAttr('disabled');
+            $("[name='"+fields[i]+"']").removeClass('field_disabled');
+        }
       } else {
-        $("[name='"+fields[i]+"']").removeAttr('disabled');
-        $("[name='"+fields[i]+"']").removeClass('field_disabled');
+        if(group[g]['default']['status']=='disable'){
+        	$("[name='"+fields[i]+"']").removeAttr('disabled');
+            $("[name='"+fields[i]+"']").removeClass('field_disabled');
+        }else{
+        	$("[name='"+fields[i]+"']").attr('disabled','disabled');
+            $("[name='"+fields[i]+"']").addClass('field_disabled');
+        }
       }
+       var field_span = $("[name='"+fields[i]+"']").closest( "span" );
+       if(group[g]['default']['display']=='yes'){
+       	var hide_class_name = field_span.attr("class");
+	        if(hide_class_name == 'field'){
+	        	field_span.show();
+	        }
+       }
+       if(group[g]['default']['display']=='no'){
+       	var hide_class_name = field_span.attr("class");
+	        if(hide_class_name == 'field'){
+	        	field_span.hide();
+	        }
+       }
+    }
+    if (typeof group[g]['condition'] !== 'undefined') {
+  	  var condition = group[g]['condition'];
+    	  if(typeof condition[v] !== 'undefined'){
+    		  var condition_fileds = condition[v];
+    		  for(var k=0;k<condition_fileds.length;k++){
+    			  if(condition_fileds[k]['status'] == 'disable'){
+	    			  $("[name='"+condition_fileds[k]['field_name']+"']").attr('disabled','disabled');
+	    		        $("[name='"+condition_fileds[k]['field_name']+"']").addClass('field_disabled');
+	    		  }else{
+	    			  $("[name='"+condition_fileds[k]['field_name']+"']").removeAttr('disabled');
+	    		        $("[name='"+condition_fileds[k]['field_name']+"']").removeClass('field_disabled');
+	    		  }
+    			  var field_span = $("[name='"+condition_fileds[k]['field_name']+"']").closest( "span" );
+    			  if(condition_fileds[k]['display']=='yes'){
+    		        	var hide_class_name = field_span.attr("class");
+    			        if(hide_class_name == 'field'){
+    			        	field_span.show();
+    			        }
+    		        }
+    			  if(condition_fileds[k]['display']=='no'){
+    		        	var hide_class_name = field_span.attr("class");
+    			        if(hide_class_name == 'field'){
+    			        	field_span.hide();
+    			        }
+    		        }
+    		  }
+    		  
+    	  }
+     }
+    for(var i=0;i<fields.length;i++) {
+    	var field = $("[name='"+fields[i]+"']");
+    	if(field.length != 0 && typeof field.attr('onchange') !== 'undefined' && field.prop('tagName').toLowerCase() =='select'){
+    		if(field.is(":visible") && !field.is(":disabled")){
+    			field.trigger('change');
+    		}else{
+    			if(field.is(":disabled")){
+    				safe_disabled(fields[i], 'none');
+    			}
+    		}
+    	}
     }
   }
 }
-
+function safe_hide(g, v) {
+	  if (typeof group[g]['fields']!== 'undefined') {
+	    var fields = group[g]['fields'];
+	    for(var i=0;i<fields.length;i++) {
+	      if (v == 'false') {
+	        $("[name='"+fields[i]+"']").attr('disabled','disabled');
+	        $("[name='"+fields[i]+"']").addClass('field_disabled');
+	        var field_tr = $("[name='"+fields[i]+"']").closest( "tr" );
+	        field_tr.hide();
+	        var hide_class_name = field_tr.attr("class");
+	        if(hide_class_name != ''){
+	        	field_tr.attr('class', '');
+		        field_tr.attr('tmpclass', hide_class_name);
+	        }
+	      } else {
+	    	$("[name='"+fields[i]+"']").removeAttr('disabled');
+  	        $("[name='"+fields[i]+"']").removeClass('field_disabled');
+    		var field_tr = $("[name='"+fields[i]+"']").closest( "tr" );
+    		field_tr.show();  
+    		var hide_class_name = field_tr.attr("tmpclass");
+	        field_tr.attr('class', hide_class_name);
+	      }
+	    }
+	    for(var i=0;i<fields.length;i++) {
+	    	var field = $("[name='"+fields[i]+"']");
+	    	if(typeof field.attr('onchange') !== 'undefined' && field.prop('tagName').toLowerCase() =='select'){
+	    		if(field.is(":visible") && !field.is(":disabled")){
+	    			field.trigger('change');
+	    		}
+	    		
+	    	}
+	    }
+	  }
+	}
+function set_group(group , action ){
+	if(action == 'disable'){
+		safe_hide(group, 'false');
+	}else{
+		safe_hide(group, 'true');
+	}
+}
+function safe_condition_disabled(g, v) {
+	  if (typeof group_condition[g]!=='undefined') {
+		  var condition = group_condition[g];
+		  if(typeof condition['default'] !== 'undefined'){
+			  for(var i=0;i<condition['groups'].length;i++) {
+				  set_group(condition['groups'][i] , condition['default']);
+			  }
+		  }
+		  if(typeof condition['condition'][v] !== 'undefined'){
+			  $.each(condition['condition'][v], function(index, value) {
+				  set_group(index , value);
+			  });
+		  }
+	  }
+	}
 /**
  * @function safe_disabled_field_init
  * @param
@@ -31,8 +150,10 @@ function safe_disabled(g, v) {
 function safe_disabled_field_init() {
   $('form').find('input,select,textarea').each(function(){
       if(this.tagName.toLowerCase() == 'select') {
-	      if($(this).attr('onchange')!=undefined){
-	    	  $(this).trigger('change');
+          if(typeof $(this).attr('onchange') !== 'undefined'){
+	          if($(this).is(":visible") && !$(this).is(":disabled")){
+	              $(this).trigger('change');
+	          }
 	      }
       }
   });
@@ -111,7 +232,12 @@ function safe_enable_datepicker(){
 
 }
 
-function safe_toggle_fields(header_id, fields_class){
+function safe_toggle_fields(header_id, fields_class, setting){
+  if(typeof setting !== 'undefined'){
+	  setting = 0; 
+  }else{
+	  setting = 'slow';
+  }
   if($('#'+header_id).hasClass('header_collapse')){
     $('#'+header_id).removeClass('header_collapse');
     $('#'+header_id).addClass('header_expand');
@@ -120,7 +246,7 @@ function safe_toggle_fields(header_id, fields_class){
     $('#'+header_id).addClass('header_collapse');
   }
   $('.'+fields_class).each(function(){
-    $(this).fadeToggle('slow');
+    $(this).fadeToggle(setting);
   });
 };
 
@@ -148,14 +274,35 @@ function safe_handle_create_button(button_obj){
 					  var ok_data = new Object();//{ "delete": "delete"};
 					  //get submit button data from confime page
 					  if(typeof rs.action_url !== 'undefined'){
-						  if (safe_create_box(rs.page_title,rs.content,rs.action_url,'',rs.action_method,'',ok_data,null ,ok_name)){
+					      if(typeof rs.setting !== 'undefined'){
+                              setting = rs.setting;
+                          }else{
+                              setting = false;
+                          }
+						  if (safe_create_box(rs.page_title,rs.content,rs.action_url,'',rs.action_method,'',ok_data,null ,ok_name,setting)){
 						  }else{
 							  throw {'title':rs.page_title, 'content':'Open confirm dialog failed.', 'type':'alert'};
 						  }
 					  }else{
-						  throw {'title':rs.page_title, 'content':'action_url error!', 'type':'alert'};
+						  if (typeof rs.redirect_url !== 'undefined'){
+							  //if redirect url returned no need show confirm dialog
+        					  window.location = rs.redirect_url;
+        				  }else{
+        					  throw {'title':rs.page_title, 'content':'action_url error!', 'type':'alert'};
+        				  }
 					  }  
 				   }else{
+					  var tmp_reason = '';
+ 					  if(typeof rs.reason !== 'undefined' && rs.reason !== null){
+ 						  if (rs.reason instanceof Array){
+ 							  tmp_reason = rs.reason.join('<br>');
+ 						  }else{
+ 							  tmp_reason = rs.reason;
+ 						  }
+ 					  }
+ 					  if (tmp_reason != ''){
+ 						  rs.message = rs.message + '<br>' + tmp_reason;
+ 					  }
 					 throw {'title':rs.page_title, 'content':rs.message, 'type': rs.message_type};
 				   };
 			  }catch(err){
@@ -164,7 +311,11 @@ function safe_handle_create_button(button_obj){
 			  
 		  },
 		  error:function(xhr){
-			safe_message_box('An error occurred:', safe_message_style(xhr.status + " " + xhr.statusText,'alert'));
+			  if(xhr.status == '200'){
+				  location.reload(true);
+			  }else{
+				  safe_message_box('An error occurred:', safe_message_style(xhr.status + " " + xhr.statusText,'alert'));
+			  }
 		  }
 		});
 }
@@ -189,7 +340,16 @@ function safe_handle_confirm_button(button_obj){
 					  var ok_data = new Object();//{ "delete": "delete"};
 					  //get submit button data from confime page
 					  if(typeof rs.action_url !== 'undefined'){
-						  if (safe_confirmation_box(rs.page_title,safe_message_style(rs.message,rs.message_type),rs.action_url,'','POST','',ok_data)){
+						  var ok_name ='Ok';
+      					  if(typeof rs.buttons !== 'undefined'){
+      						  ok_name = rs.buttons[0].value;
+      					  }
+      					  if(typeof rs.setting !== 'undefined'){
+      						  setting = rs.setting;
+    					  }else{
+    						  setting = false;
+    					  }
+						  if (safe_confirmation_box(rs.page_title,safe_message_style(rs.message,rs.message_type),rs.action_url,'','POST','',ok_data,null,ok_name,setting)){
 						  }else{
 							  throw {'title':rs.page_title, 'content':'Open confirm dialog failed.', 'type':'alert'};
 						  }
@@ -197,7 +357,18 @@ function safe_handle_confirm_button(button_obj){
 						  throw {'title':rs.page_title, 'content':'action_url error!', 'type':'alert'};
 					  }  
 				   }else{
-					 throw {'title':rs.page_title, 'content':rs.message, 'type': rs.message_type};
+					   var tmp_reason = '';
+	 					  if(typeof rs.reason !== 'undefined' && rs.reason !== null){
+	 						  if (rs.reason instanceof Array){
+	 							  tmp_reason = rs.reason.join('<br>');
+	 						  }else{
+	 							  tmp_reason = rs.reason;
+	 						  }
+	 					  }
+	 					  if (tmp_reason != ''){
+	 						  rs.message = rs.message + '<br>' + tmp_reason;
+	 					  }
+					   throw {'title':rs.page_title, 'content':rs.message, 'type': rs.message_type};
 				   };
 			  }catch(err){
 				  safe_message_box(err.title,safe_message_style(err.content,err.type));
@@ -205,7 +376,11 @@ function safe_handle_confirm_button(button_obj){
 			  
 		  },
 		  error:function(xhr){
-			safe_message_box('An error occurred:', safe_message_style(xhr.status + " " + xhr.statusText,'alert'));
+			  if(xhr.status == '200'){
+				  location.reload(true);
+			  }else{
+				  safe_message_box('An error occurred:', safe_message_style(xhr.status + " " + xhr.statusText,'alert'));
+			  }
 		  }
 		});
 }
@@ -231,7 +406,7 @@ function defaultvalue(arg, val) {
  * @param cancel_data (click cancel send http requst with this cancel_data)
  * @desc show a confirm dialog box with ok or cancel button
  */
-function safe_confirmation_box(title, content, url_ok, url_cancel, ok_method, cancel_method, ok_data, cancel_data) {
+function safe_confirmation_box(title, content, url_ok, url_cancel, ok_method, cancel_method, ok_data, cancel_data, ok_name,setting) {
 	
 	title 		= defaultvalue(title, '');
 	content 	= defaultvalue(content, '');
@@ -241,6 +416,8 @@ function safe_confirmation_box(title, content, url_ok, url_cancel, ok_method, ca
 	cancel_method 	= cancel_method != 'POST' ? 'GET' : 'POST';
 	ok_data		= defaultvalue(ok_data, null);
 	cancel_data = defaultvalue(cancel_data, null);
+	ok_name		= defaultvalue(ok_name, 'Ok');
+	setting		= defaultvalue(setting, '');
 	
 	if (title != '' && content != '' && url_ok != '' ){
 		return $("<div>" + content + "</div>").dialog({
@@ -255,44 +432,65 @@ function safe_confirmation_box(title, content, url_ok, url_cancel, ok_method, ca
 			resizable: false,
 			dialogClass: 'msgbox',
 			title: title,
-			buttons: {
-				"Ok": function() {
-					$(this).dialog("close");
-					//ok_data.return_type = 'json';//default return json
-					cancel_refresh();
-					ui_ajax_running = $.ajax({
-	            		  type: ok_method,
-	            		  url: url_ok,
-	            		  data : ok_data,
-	            		  dataType: 'json',
-	            		  async:false,
-	            		  success:function(rs){
-	            			//handle result
-	            			  if (rs.result == true ){
-	            				  //return a url and redirect url
-	            				  if (typeof rs.redirect_url !== 'undefined'){
-	            					  window.location = rs.redirect_url;
-	            				  }else{
-	            					  safe_message_box('An error occurred:', safe_message_style("Redirect url undefined.",'alert'));
-	            				  }
-	            			  }else{
-	            				  safe_message_box(rs.page_title,safe_message_style(rs.message,rs.message_type));
-	            			  }
-	            		  },
-	            		  error:function(xhr){
-	            			  safe_message_box('An error occurred:', safe_message_style(xhr.status + " " + xhr.statusText,'alert'));
-	            		  }
-	            		});
-	                return false;
-	            },
-				"Cancel": function() {
-					if (url_cancel != ''){
-						//do some thing
-					}
-					$(this).dialog("close");
-	                return false;
-	            }
-			}
+			buttons:[ 
+				{
+					text : ok_name,
+					click:function() {
+						//show process bar
+						if(setting){
+							p_title = defaultvalue(setting['title'], title);
+							p_sec = defaultvalue(setting['sec'], 30);
+							p_msg = defaultvalue(setting['msg'], 'Timeout ...');
+							p_label = defaultvalue(setting['label'], 'Ok');
+							p_url = defaultvalue(setting['url'], '');
+							processing_ui = safe_wait(p_title,'spinning_wheel',p_sec,p_msg,p_label,p_url);
+						}
+						
+						$(this).dialog("close");
+						cancel_refresh();
+						var timeout = window.setTimeout(function() {
+							ui_ajax_running = $.ajax({
+			            		  type: ok_method,
+			            		  url: url_ok,
+			            		  data : ok_data,
+			            		  dataType: 'json',
+			            		  async:true,
+			            		  success:function(rs){
+			            			//handle result
+			            			  if(processing_ui) {
+			            				  processing_ui.dialog("close");
+			            			  }
+			            			  if (rs.result == true ){
+			            				  //return a url and redirect url
+			            				  if (typeof rs.redirect_url !== 'undefined'){
+			            					  window.location = rs.redirect_url;
+			            				  }else{
+			            					  safe_message_box('An error occurred:', safe_message_style("Redirect url undefined.",'alert'));
+			            				  }
+			            			  }else{
+			            				  safe_message_box(rs.page_title,safe_message_style(rs.message,rs.message_type));
+			            			  }
+			            		  },
+			            		  error:function(xhr){
+			            			  processing_ui.dialog("close");
+			            			  safe_message_box('An error occurred:', safe_message_style(xhr.status + " " + xhr.statusText,'alert'));
+			            		  }
+			            		});
+						}, 200);
+		                return false;
+		            }
+				},
+				{
+					text : 'Cancel',
+					click:function() {
+						if (url_cancel != ''){
+							//do some thing
+						}
+						$(this).dialog("close");
+		                return false;
+		            }
+				}
+			]
 		});
 	}else{
 		safe_message_box('An error occurred:', safe_message_style("Parameter missing, Open the confirmation box failed",'alert'));
@@ -303,14 +501,20 @@ function safe_confirmation_box(title, content, url_ok, url_cancel, ok_method, ca
  * @function safe_security_timer
  * @param sec (Value in second defining maximum time to wait for page to complete - Default is 10sec - 0 disable mechanism)
  * @param msg (Message to display when timeout reached)
- * @param lable (Button label for redirection )
+ * @param label (Button label for redirection )
  * @param url ( URL to redirect user to after timeout expired when user press the redirect button - default is proper $_SERVER array entry containing base url + port ) 
  * @desc When the timeout fires we must add in the waiting dialog a custom message and a redirect button 
  */
-function safe_security_timer(sec,msg,lable,url){
+function safe_security_timer(sec,msg,label,url,wait_dialog){
 	sec 	= defaultvalue(sec, 0);
+	wait_dialog    = defaultvalue(wait_dialog, null);
 	if(sec > 0){
-		return setTimeout(function(){ timeout_dialog = safe_message_box('Timeout', msg,null,lable,function() {window.location = url;});}, sec*1000);
+		return setTimeout(function(){ 
+		    if(wait_dialog != null){
+		        wait_dialog.dialog("close");//close the processing dialog box
+		    }
+		    timeout_dialog = safe_message_box('Timeout', msg,null,label,function() {if(url!='') {window.location = url;}else{$(this).dialog("close");}});
+		    }, sec*1000);
 	}
 	return false;
 }
@@ -321,16 +525,16 @@ function safe_security_timer(sec,msg,lable,url){
  * @param type (spinning_wheel, or processing_bar ) 
  * @desc show a please wait dialog and block the ui 
  */
-function safe_wait(content,type,sec,msg,lable,url){
+function safe_wait(content,type,sec,msg,label,url){
 	content 	= defaultvalue(content, 'Please wait...');
 	type 	= defaultvalue(type, 'spinning_wheel');
 	sec 	= defaultvalue(sec, 0);
-	msg 	= defaultvalue(msg, '');
-	lable 	= defaultvalue(lable, '');
+	msg 	= defaultvalue(msg, 'Timeout...');
+	label 	= defaultvalue(label, 'Ok');
 	url 	= defaultvalue(url, '');
 	if(type == 'spinning_wheel'){
-		var security_timer = safe_security_timer(sec,msg,lable,url);
-		return $('<div><center><br><img src="/templates/base/images/icons/16x16/icon-loading.gif" style="vertical-align:middle;"> &nbsp;Please wait... <br><br><div id="content">' + content + "</div></center></div>").dialog({
+		
+		var wait_dialog = $('<div><center><br><img src="/templates/base/images/icons/16x16/icon-loading.gif" style="vertical-align:middle;"> &nbsp;Please wait... <br><br><div id="content">' + content + "</div></center></div>").dialog({
 			autoOpen: true,
 			position: "center",
 			modal: true,
@@ -351,6 +555,8 @@ function safe_wait(content,type,sec,msg,lable,url){
 				$(this).dialog("destroy").remove();
 			}
 		});
+		var security_timer = safe_security_timer(sec,msg,label,url,wait_dialog);
+		return wait_dialog;
 	}else if(type == 'processing_bar'){
 		return $('<div><center><br><img src="/templates/base/images/icons/16x16/icon-loading.gif" style="vertical-align:middle;"> &nbsp;Please wait... <br><br><div id="safe_progress_bar"><span id="percent" style="width: 100%;"></span></div><br><div id="content">' + content + "</div></center></div>").dialog({
 			autoOpen: true,
@@ -395,6 +601,23 @@ function safe_progressbar_update(bar,value){
 		bar.find(".ui-progressbar-value").css('opacity','');
 	}
 }
+function obj_loop(object){
+    var collection= '', next, item;
+    for(item in object){
+        if(object.hasOwnProperty(item)){
+            next= object[item];
+            if(typeof next == 'object' && next != null){
+                collection = collection +   obj_loop(next) ;
+            }else{
+            	collection = collection + '<li style="list-style-type:none;">'+String(next) + '</li>';
+            }
+        }
+    }
+    if(collection != ''){
+    	collection = '<ul style="list-style-type:none;  padding-left: 10px; margin: 0px;" >' +  collection + '</ul>';
+    }
+    return collection;
+}
 /**
  * @function safe_create_box
  * @param title	(confirm box title)
@@ -408,7 +631,7 @@ function safe_progressbar_update(bar,value){
  * @param ok_name (ok button name)
  * @desc show a confirm dialog box with ok or cancel button
  */
-function safe_create_box(title, content, url_ok, url_cancel, ok_method, cancel_method, ok_data, cancel_data,ok_name) {
+function safe_create_box(title, content, url_ok, url_cancel, ok_method, cancel_method, ok_data, cancel_data,ok_name,setting) {
 	
 	title 		= defaultvalue(title, '');
 	content 	= defaultvalue(content, '');
@@ -419,6 +642,7 @@ function safe_create_box(title, content, url_ok, url_cancel, ok_method, cancel_m
 	ok_data		= defaultvalue(ok_data, null);
 	cancel_data = defaultvalue(cancel_data, null);
 	ok_name		= defaultvalue(ok_name, 'Create');
+	setting     = defaultvalue(setting, null);//wait dialog settings
 		
 	if (title != '' && content != '' && url_ok != '' ){
 		return $("<div>" + content + "</div>").dialog({
@@ -488,7 +712,7 @@ function safe_create_box(title, content, url_ok, url_cancel, ok_method, cancel_m
 							    	$("#safe_progress_bar .ui-progressbar-value").animate({width: percentComplete+"%"},50);
 							        $("#safe_progress_bar").find("#percent").html(percentComplete + '%')
 							        if(percentComplete >= 100){
-							        	upload_processing_ui.find("#content").html('File Uploaded Successfully! System Processing ...');
+							        	upload_processing_ui.find("#content").html('File Uploaded successfully! System processing...');
 							        }
 							    },
 						        success:       function(rs){
@@ -496,7 +720,11 @@ function safe_create_box(title, content, url_ok, url_cancel, ok_method, cancel_m
 			            			  if (rs.result == true ){
 			            				  //add successful and return a redirect url
 			            				  if (typeof rs.redirect_url !== 'undefined'){
-			            					  window.location = rs.redirect_url;
+			            					  if(rs.redirect_method=='POST'){
+			            						  redirect_by_post(rs.redirect_url,rs.post_data);  
+			            					  }else{
+			            						  window.location = rs.redirect_url;
+			            					  }
 			            				  }else{
 			            					  safe_message_box('An error occurred:', safe_message_style("Redirect url undefined.",'alert'));
 			            				  }
@@ -507,7 +735,11 @@ function safe_create_box(title, content, url_ok, url_cancel, ok_method, cancel_m
 			            					  //show the form again with validation error
 				            				  ok_data = null;//clear ok_data
 				          					  if(typeof rs.action_url !== 'undefined'){
-				          						  if (safe_create_box(rs.page_title,rs.content,rs.action_url,'',rs.action_method,'',ok_data)){
+				          						  var ok_name ='Create';
+					          					  if(typeof rs.buttons !== 'undefined'){
+					          						  ok_name = rs.buttons[0].value;
+					          					  }
+				          						  if (safe_create_box(rs.page_title,rs.content,rs.action_url,'',rs.action_method,'',ok_data, null,ok_name,setting)){
 				          							  old_dialog.dialog("close");//close the old dialog
 				          							  old_dialog.dialog( "destroy" ).remove();//distory the old add dialog
 				          						  }else{
@@ -525,13 +757,28 @@ function safe_create_box(title, content, url_ok, url_cancel, ok_method, cancel_m
 			            				             }
 			            					  	};
 			            					  //show other error dialog in a new message box
+			            					  var tmp_reason = '';
+			            					  if(typeof rs.reason !== 'undefined' && rs.reason !== null){
+			            						  if(typeof rs.reason === 'object'){
+			            							  tmp_reason = obj_loop(rs.reason);
+			            						  }else if(typeof rs.reason === 'string'){
+			            							  tmp_reason = rs.reason;
+			            						  }
+			            					  }
+			            					  if (tmp_reason != ''){
+			            						  rs.message = rs.message + '<br>' + tmp_reason;
+			            					  }
 			            					  safe_message_box(rs.page_title,safe_message_style(rs.message,rs.message_type),close_method);
 			            					 
 			            				  }
 			            			  }
 			            		},  // post-submit callback
 			            		error:function(xhr){
-			              			safe_message_box('An error occurred:', safe_message_style(xhr.status + " " + xhr.statusText,'alert'));
+			            			if(xhr.status == '200'){
+			          				  	location.reload(true);
+			          			  	}else{
+			          			  		safe_message_box('An error occurred:', safe_message_style(xhr.status + " " + xhr.statusText,'alert'));
+			          			  	}
 			              		},
 			              		complete: function(xhr) {
 			              				//status.html(xhr.responseText);
@@ -559,29 +806,53 @@ function safe_create_box(title, content, url_ok, url_cancel, ok_method, cancel_m
 						$(forms[0]).submit();
 					}else{
 						cancel_refresh();
+						
 						ui_ajax_running = $.ajax({
 		            		  type: ok_method,
 		            		  url: url_ok,
 		            		  data : ok_data,
 		            		  dataType: 'json',
-		            		  async:false,
+		            		  async:true,
+		            		  beforeSend: function() {
+                                  old_dialog.dialog("close");//hide the old dialog box
+                                  //add wait dialog
+                                  if(setting){
+                                      p_title = defaultvalue(setting['title'], title);
+                                      p_sec = defaultvalue(setting['sec'], 30);
+                                      p_msg = defaultvalue(setting['msg'], 'Timeout ...');
+                                      p_label = defaultvalue(setting['label'], 'Ok');
+                                      p_url = defaultvalue(setting['url'], '');
+                                      processing_ui = safe_wait(p_title,'spinning_wheel',p_sec,p_msg,p_label,p_url);
+                                  }
+                              },
 		            		  success:function(rs){
+		            		      if(processing_ui) {
+                                      processing_ui.dialog("close");
+                                  }
 		            			//handle result
-		            			  if (rs.result == true ){
+		            			  if (rs.result == true && typeof rs.close_dialog == 'undefined' ){
 		            				  //add successful and return a redirect url
 		            				  if (typeof rs.redirect_url !== 'undefined'){
-		            					  window.location = rs.redirect_url;
+		            					  if(rs.redirect_method=='POST'){
+		            						  redirect_by_post(rs.redirect_url,rs.post_data);  
+		            					  }else{
+		            						  window.location = rs.redirect_url;
+		            					  }
 		            				  }else{
 		            					  safe_message_box('An error occurred:', safe_message_style("Redirect url undefined.",'alert'));
 		            				  }
 		            			  }else{
 		            				  //add fail
-		            				  if(rs.reload==true){
+		            				  if(!rs.result && rs.reload==true){
 		            					  //uer input data run_validation fail
 		            					  //show the form again with validation error
 			            				  ok_data = null;//clear ok_data
 			          					  if(typeof rs.action_url !== 'undefined'){
-			          						  if (safe_create_box(rs.page_title,rs.content,rs.action_url,'',rs.action_method,'',ok_data)){
+			          						  var ok_name ='Create';
+				          					  if(typeof rs.buttons !== 'undefined'){
+				          						  ok_name = rs.buttons[0].value;
+				          					  }
+			          						  if (safe_create_box(rs.page_title,rs.content,rs.action_url,'',rs.action_method,'',ok_data,null,ok_name,setting)){
 			          							  old_dialog.dialog("close");//close the old dialog
 			          							  old_dialog.dialog( "destroy" ).remove();//distory the old add dialog
 			          						  }else{
@@ -591,21 +862,45 @@ function safe_create_box(title, content, url_ok, url_cancel, ok_method, cancel_m
 			          						  throw {'title':rs.page_title, 'content':'action_url error!', 'type':'alert'};
 			          					  }  
 		            				  }else{
+		            					  if(typeof rs.close_dialog !== 'undefined' && rs.close_dialog == true){
+		            					      old_dialog.dialog("close");//hide the old dialog box
+		            					      old_dialog.dialog( "destroy" ).remove();//distory the old add dialog
+		            					      var close_method = null;
+		            					  }else{
+		            					      old_dialog.dialog("close");//hide the old dialog box
+	                                          var close_method = {
+	                                                "dialogbeforeclose": function() {
+	                                                    old_dialog.dialog("open");//show the old dialog box when close the error message box
+	                                                 }
+	                                            };
+		            					  }
 		            					  
-		            					  old_dialog.dialog("close");//hide the old dialog box
-		            					  var close_method = {
-		            				            "dialogbeforeclose": function() {
-		            				            	old_dialog.dialog("open");//show the old dialog box when close the error message box
-		            				             }
-		            					  	};
+		            					  //show other error dialog in a new message box
+		            					  var tmp_reason = '';
+		            					  if(typeof rs.reason !== 'undefined' && rs.reason !== null){
+		            						  if (rs.reason instanceof Array){
+		            							  tmp_reason = rs.reason.join('<br>');
+		            						  }else{
+		            							  tmp_reason = rs.reason;
+		            						  }
+		            					  }
+		            					  if (tmp_reason != ''){
+		            						  rs.message = rs.message + '<br>' + tmp_reason;
+		            					  }
 		            					  //show other error dialog in a new message box
 		            					  safe_message_box(rs.page_title,safe_message_style(rs.message,rs.message_type),close_method);
-		            					 
 		            				  }
 		            			  }
 		            		  },
 		            		  error:function(xhr){
-		            			  safe_message_box('An error occurred:', safe_message_style(xhr.status + " " + xhr.statusText,'alert'));
+		            		      if(processing_ui) {
+                                      processing_ui.dialog("close");
+                                  }
+		            			  if(xhr.status == '200'){
+		            				  location.reload(true);
+		            			  }else{
+		            				  safe_message_box('An error occurred:', safe_message_style(xhr.status + " " + xhr.statusText,'alert'));
+		            			  }
 		            		  }
 		            		});
 					}
@@ -639,12 +934,16 @@ function safe_message_style(content,type){
 	content 	= defaultvalue(content, '');
 	type 		= defaultvalue(type, 'info');//alert
 	var tmp_string ='';
+	 if (content instanceof Array){
+         tmp_reason = content.join('<br>');
+         content = tmp_reason;
+     }
 	if(type == 'alert'){
-		tmp_string ='<div style="margin-top: 10px; margin-bottom: 10px;"><div style="float: left; margin: 0 7px 20px 0;"><span class="ui-icon ui-icon-alert">-</span></div>'+content+'</div>';
+		tmp_string ='<div style="margin-top: 10px; margin-bottom: 10px;"><div style="float: left; margin: 0 2px 2px 0;"><span class="ui-icon ui-icon-alert">-</span></div><div style="margin-left: 20px;">'+content+'</div></div>';
 	}else if(type == 'notice'){
-		tmp_string ='<div style="margin-top: 10px; margin-bottom: 10px;"><div style="float: left; margin: 0 7px 20px 0;"><span class="ui-icon ui-icon-notice">-</span></div>'+content+'</div>';
+		tmp_string ='<div style="margin-top: 10px; margin-bottom: 10px;"><div style="float: left; margin: 0 2px 2px 0;"><span class="ui-icon ui-icon-notice">-</span></div><div style="margin-left: 20px;">'+content+'</div></div>';
 	}else{
-		tmp_string ='<div style="margin-top: 10px; margin-bottom: 10px;"><div style="float: left; margin: 0 7px 20px 0;"><span class="ui-icon ui-icon-info">-</span></div>'+content+'</div>';
+		tmp_string ='<div style="margin-top: 10px; margin-bottom: 10px;"><div style="float: left; margin: 0 2px 2px 0;"><span class="ui-icon ui-icon-info">-</span></div><div style="margin-left: 20px;">'+content+'</div></div>';
 	}
 	return tmp_string;
 }
@@ -700,7 +999,9 @@ function safe_message_box(title, content,close_method,btn_name,btn_method){
 	if(typeof $.ui.dialog.prototype.oldOpen === 'undefined'){
 		var dialogExtensions ={
 		        oldOpen: $.ui.dialog.prototype.open,
+		        oldClose: $.ui.dialog.prototype.close,
 		        open: function(event){
+		        	dialog_open = true;
 		        	if(this.options.position !== false && this.options.position=='center'){
 		        		var window_width = this.window.innerWidth();
 		        		var window_height = this.window.innerHeight();
@@ -721,9 +1022,14 @@ function safe_message_box(title, content,close_method,btn_name,btn_method){
 		        		this.widget().height(this.options.maxHeight);
 		        		$(this.widget()).find('.ui-dialog-content').first().height(this.options.maxHeight-85);//title and foot bar 85
 		        	}
-		        } 
-		    };
-		    $.extend($.ui.dialog.prototype, dialogExtensions);
+		        },
+			close:function(event){
+				this.oldClose(event);
+				//value sett too fast
+				setTimeout(function(){dialog_open = false;},500);
+			} 
+		};
+		$.extend($.ui.dialog.prototype, dialogExtensions);
 	}
 })(jQuery);
 
@@ -952,8 +1258,49 @@ function system_status_fresh(init,id){
 		timeout_obj=setTimeout(function(){ajax_running = eval(tmp_function_name);},5000);
 	}
 }
+
+/**
+ * @function redirect_by_post
+ * @param
+ * @desc redirect url by post with some post data
+ */
+function redirect_by_post(url,data){
+	if(typeof url !== 'undefined' && url != ''){
+		var form = document.createElement("form");
+	    form.setAttribute("method", "post");
+	    form.setAttribute("action", url);
+	    if(typeof data !== 'undefined'){
+	    	for(var key in data) 
+	        { 
+	    		if (data[key].constructor === Array){
+	    			for(var subkey in data[key])
+	    			{
+	    				var hiddenField = document.createElement("input");
+			            hiddenField.setAttribute("name", key+'[]');
+			            hiddenField.setAttribute("value",data[key][subkey] );
+			            hiddenField.setAttribute("type", "checkbox");
+			            hiddenField.setAttribute("checked", 'checked');
+			            form.appendChild(hiddenField);
+	    			}
+	    		}else{
+	    			var hiddenField = document.createElement("input");
+		            hiddenField.setAttribute("name", key);
+		            hiddenField.setAttribute("value",data[key] );
+		            hiddenField.setAttribute("type", "hidden");
+		            form.appendChild(hiddenField);
+	    		}
+	        	
+	            
+	        }
+	    }
+	    document.body.appendChild(form);
+	    form.submit();
+	}
+}
+
 var p = 0;
 var upload_processing_ui = null;
+var processing_ui = null;
 
 (function($){
 	if(system_status_timer === null){
@@ -975,3 +1322,14 @@ var upload_processing_ui = null;
 		}).tooltip();
 	}
 })(jQuery);
+var dialog_open = false;
+$(document).ready(function() {
+    $('form:first *:input[type!=hidden]:first').focus();
+});
+$(document).ready(function() {
+	if(typeof _category !== 'undefined'){
+		for(var i=0;i<_category.length;i++) {
+			safe_toggle_fields('safe_category_header_'+_category[i],'safe_category_'+_category[i], '0');
+		}
+	}
+});
