@@ -39,20 +39,15 @@ class Sng_snmpd_config_class extends Safe_configurable_object_class
         $this->add_enum_field('version3', 'Version', 'dropdown', 'false', $snmp_version);
 
         $this->set_field_category('assisted', 'General');
-        // Default interface name
         $def_if = 'all';
-        // Populate IP enumeration with all management interface IP
-        $mgmt_if = $this->_node->hardware()->adapters(Safe_hardware_adapter_class::ETHERNET_ADAPTER);
+        $mgmt_if = $this->_adapters();
         $if_enum = array();
         $if_enum['all'] = 'All interfaces';
-        foreach($mgmt_if as $if_name=>$if) {
-            if(strlen($if->ip_address()))
-            {
-                $if_enum[$if->ip_address()] = $if->name() . ' - ' . $if->ip_address();
-            }
+        foreach($mgmt_if as $if_name => $if_ip) {
+            $if_enum[$if_name] = $if_name . ' - ' . $if_ip;
         }
-        $this->add_enum_field('ip', 'Network Interface', 'dropdown', '', $if_enum);
-        $this->set_field_help('ip', 'Select interface that snmpd listens on.');
+        $this->add_enum_field('interface', 'Network Interface', 'dropdown', '', $if_enum);
+        $this->set_field_help('interface', 'Select interface that SNMP listens on.');
         // protocol
         $transport = array(
             'udp' => 'UDP',
@@ -62,7 +57,7 @@ class Sng_snmpd_config_class extends Safe_configurable_object_class
         $this->add_field('port', 'Port', 'text', '161',10);
         $this->set_field_help('port', 'Port number that snmpd listens on.');
         $this->set_field_rules('port', 'required|integer|greater_than[0]|less_than[61000]|callback_port_check[port]');
-        $this->set_field_category('ip', 'Network Configuration');
+        $this->set_field_category('interface', 'Network Configuration');
 
         // User/psw
         $this->add_field('user', 'User Name', 'text', '',20);
@@ -73,7 +68,7 @@ class Sng_snmpd_config_class extends Safe_configurable_object_class
 
         // Create conditional group
         $this->create_group('options', 
-            array('ip', 'transport', 'port', 'version', 'community', 'user', 'password') );
+            array('interface', 'transport', 'port', 'version', 'community', 'user', 'password') );
         $this->conditional_control('assisted', 'options');
 
         return parent::configure();
@@ -104,7 +99,19 @@ class Sng_snmpd_config_class extends Safe_configurable_object_class
         }
         return $table_line;
     }
-    
+    private  function _adapters(){
+        if(!$this->_adapters){
+            // Populate IP enumeration with all management interface IP
+            $mgmt_if = $this->_node->hardware()->adapters(Safe_hardware_adapter_class::ETHERNET_ADAPTER);
+            foreach($mgmt_if as $if_name=>$if) {
+                if(strlen($if->ip_address()))
+                {
+                    $this->_adapters[$if->name()] = $if->ip_address();
+                }
+            }
+        }
+        return $this->_adapters;
+    }
     public function port_check($field_name = null){
         if($field_name){
             $port_name = $this->get_data_label($field_name);
@@ -113,7 +120,14 @@ class Sng_snmpd_config_class extends Safe_configurable_object_class
         }else{
             return true;
         }
-        $ip = $this->get_data_value('ip',false);
+        //adapters interfaces name to ip address
+        $tmp_adapters = $this->_adapters();
+        $if_name = $this->get_data_value('interface',false);
+        if(isset($tmp_adapters[$if_name])){
+            $ip =  $tmp_adapters[$if_name];
+        }else{
+            $ip = 'all';
+        }
         //ip port
         $ports[] = array('name' =>$port_name,'port'=>$port_value, 'ip'=>$ip, 'type' => $port_type);
     
@@ -322,6 +336,22 @@ class Sng_snmpd_service_class extends Safe_service_class
     {
         return true;
     }
+    private function interface_to_ip($interface){
+        $mgmt_if = $this->node()->hardware()->adapters(Safe_hardware_adapter_class::ETHERNET_ADAPTER);
+        $tmp_adapters = array();
+        foreach($mgmt_if as $if_name=>$if) {
+            if(strlen($if->ip_address()))
+            {
+                $tmp_adapters[$if->name()] = $if->ip_address();
+            }
+        }
+        if(isset($tmp_adapters[$interface])){
+            $ip =  $tmp_adapters[$interface];
+        }else{
+            $ip = 'all';
+        }
+        return $ip;
+    }
     /**
      * @brief Check for IP_PORT conflicts
      *
@@ -342,7 +372,8 @@ class Sng_snmpd_service_class extends Safe_service_class
         if('IP_PORT' == $resource_type){
             // get our configuration
             $data = $this->config()->get_data_values(false);
-            $ports[] = array('name' =>$this->name(),'port'=>$data['port'], 'ip'=>$data['ip'], 'type' => 'TCP');
+            $ip = $this->interface_to_ip($data['interface']);
+            $ports[] = array('name' =>$this->name(),'port'=>$data['port'], 'ip'=>$ip, 'type' => 'TCP');
             // Loop around resource data
             foreach($resource_data as $resource){
                 // TODO - Check
@@ -439,8 +470,9 @@ class Sng_snmpd_service_class extends Safe_service_class
         $snmpd_options_conf = array();
         $snmpd_options_conf[] = '# snmpd command line options';
         $listen = strtoupper($cfg_data['transport']).':';
-        if('all' != $cfg_data['ip']){
-            $listen .= $cfg_data['ip'].':';
+        $ip = $this->interface_to_ip($cfg_data['interface']);
+        if('all' != $ip){
+            $listen .= $ip.':';
         }
         $listen .= $cfg_data['port'];
 
