@@ -32,7 +32,7 @@ class Sng_update_module_class extends Safe_module_class
      */
     public function __construct($app)
     {
-        parent::__construct($app->node(), "update");
+        parent::__construct($app->node()->software(), "update");
     }
     public function get_data_settings($check_status = true)
     {
@@ -50,10 +50,37 @@ class Sng_update_module_class extends Safe_module_class
                                 'install',
                                 'delete',
                                 'view'
-                                )
+                                ),
+                        'methods' => array(
+                                'upload' => array(
+                                        'name' => 'Upload',
+                                        'description' => array( 'Upload a package file.' ),
+                                        'request' => 'POST',
+                                ),
+                                'description' => array(
+                                        'name' => 'Get Package desription',
+                                        'description' => 'Get Package desription',
+                                        'request' => 'GET',
+                                ),
+                                'install' => array(
+                                        'name' => 'Install',
+                                        'description' => 'Install the update',
+                                        'request' => 'POST',
+                                ),
+                                'status' => array(
+                                        'name' => 'Get update status',
+                                        'description' => 'Get update status',
+                                        'request' => 'GET',
+                                ),
+                                'delete' => array(
+                                        'name' => 'Delete',
+                                        'description' => array( 'Delete a package file.' ),
+                                        'request' => 'POST',
+                                ),
+                        ),
                 )
         );
-        if($check_status && !($this->can_install_data())){
+        if($check_status && !$this->can_install_data()){
             unset($data['package']['capabilities']);
             $data['package']['capabilities'] = array('delete','view');
         }
@@ -69,6 +96,7 @@ class Sng_update_module_class extends Safe_module_class
         // Set the module description
         $this->set_description("Update");
         // Register objects
+        /*
         $this->register_aggregate_object('package',
                 array(
                         'name' => 'Package',
@@ -90,13 +118,14 @@ class Sng_update_module_class extends Safe_module_class
                                         'request' => 'GET',
                                 ),
                                 'upload' => array(
-                                        'name' => 'Upload an Package',
+                                        'name' => 'Upload a Package',
                                         'description' => 'Upload an update',
                                         'request' => 'POST',
                                 ),
                         ),
                 )
         );
+        */
         return parent::configure();
     }
     /**
@@ -134,6 +163,24 @@ class Sng_update_module_class extends Safe_module_class
         }
         return false;
     }
+    public function info($data_name = null, $param = null, &$output){
+        // Check if specified update package can be install
+        $package_data = array();
+        $rs = $this->api_description_package($data_name, $param, $package_data);
+        if($rs){
+            //make sure all keys are lowercase and no space (replace by '_')
+            $tmp = array();
+            foreach($package_data as $key => $value){
+                $tmp[strtolower(str_replace(' ','_',$key))] = $value;
+            }
+            $tmp['type'] = 'update';
+            $tmp['data_name'] = $data_name;
+            $tmp['data'] = $param;
+            return $tmp;
+        }
+        $output[] = 'Fail to get package description !';
+        return false;
+    }
     public function log(&$output = null){
         $data_cfg = $this->get_data_settings();
         $data_cfg = $data_cfg['package'];
@@ -156,20 +203,23 @@ class Sng_update_module_class extends Safe_module_class
     }
     public function install_data($data_name = null, $data_key = null, &$output = null)
     {
-        return $this->api_install_package($data_name, null, $output);
+        return $this->api_install_package($data_name, $data_key, $output);
     }
-    public function can_install_data($data_name = null, $data_key = null, &$output = null)
-    {
-        $rs = $this->api_status_package($this->name(), null, $output);
+    public function can_install_data($info = null, &$output = null)
+    {        
+        $output_data = array();
+        $rs = $this->api_status_package($info['data_name'], $info['data'], $output_data);
         # 0 - Completed success
         # 1 - Running
         # -1 - Not Initialized/Dead
         # -2 - Not started //254
         if($rs == 254){
             return true;
-        }else{
-            return false;
         }
+        if(isset($output)){
+            $output = $output_data;
+        }
+        return false;
     }
     public function initialize_data($data_name = null, $data_key = null, &$output = null)
     {
@@ -237,26 +287,30 @@ class Sng_update_module_class extends Safe_module_class
      * @return
      */
     public function api_install_package($name, $data = null, &$output = null){
-        $data_cfg = $this->get_data_settings();
-        $data_cfg = $data_cfg['package'];
-        $file = $data_cfg['pattern'];
-        $ext  = pathinfo($file);
-        $os = $this->node()->os();
-        $file_list = $os->search_files($file);
-        if($file_list)
-        {
-            $rc = $os->execute($ext['dirname'].'/tmp/update/scripts/update.sh start ', $output, true, true);
-            if($rc){
-                sleep(1);
-                do {
-                    $os->execute($ext['dirname'].'/tmp/update/scripts/update.sh status ', $output, true, true, &$status, false);
-                    sleep(0.5);
-                } while(1 == $status);
-                if(0 != $status){
-                    error_log('ERROR - Install Package returns error <'.$status.'>');
-                    error_log('ERROR - Install Package Output='.print_r($output, true));
+        $info = $this->info($name, $data, $output);
+        $the_product = Safe_get_product();
+        if($info && $the_product->can_install_data($info, $output)){
+            $data_cfg = $this->get_data_settings();
+            $data_cfg = $data_cfg['package'];
+            $file = $data_cfg['pattern'];
+            $ext  = pathinfo($file);
+            $os = $this->node()->os();
+            $file_list = $os->search_files($file);
+            if($file_list)
+            {
+                $rc = $os->execute($ext['dirname'].'/tmp/update/scripts/update.sh start ', $output, true, true);
+                if($rc){
+                    sleep(1);
+                    do {
+                        $os->execute($ext['dirname'].'/tmp/update/scripts/update.sh status ', $output, true, true, &$status, false);
+                        sleep(0.5);
+                    } while(1 == $status);
+                    if(0 != $status){
+                        error_log('ERROR - Install Package returns error <'.$status.'>');
+                        error_log('ERROR - Install Package Output='.print_r($output, true));
+                    }
+                    return ($status==0)?true:false;
                 }
-                return ($status==0)?true:false;
             }
         }
         return false;
@@ -286,7 +340,7 @@ class Sng_update_module_class extends Safe_module_class
      *
      * @return
      */
-    public function api_status_package($name, $data = null, &$output = null){
+    public function api_status_package($name = null, $data = null, &$output = null){
         $data_cfg = $this->get_data_settings(false);
         $data_cfg = $data_cfg['package'];
         $file = $data_cfg['pattern'];
@@ -312,11 +366,19 @@ class Sng_update_module_class extends Safe_module_class
      * @return
      */
     public function api_upload_package($name, $data = null, &$output = null){
-        //upload file
-        // untar package
-        //Initialize archive using <root>/update/scripts/update.sh initialize
+        $data_def = $this->get_data_settings();
+        $upload_api_method  = $data_def['package']['methods']['upload']['request'];
+        if($upload_api_method){
+            //initialize_data called in  upload_data() 
+            return $this->upload_data('package', $name , $output , $upload_api_method);
+        }else{
+            $output['message'] = 'Upload request method error.';
+            return false;
+        }
+    }
+    public function api_delete_package($name=null, $data = null, &$output = null){
+        return $this->delete_data('package', $name);
     }
-
     public function get_data_file_content_by_key($data_name, $key, $line_count = NULL, $max_size = NULL)
     {
         $output = array();
