@@ -78,7 +78,7 @@ class Sng_data_manager extends Safe_CI_Controller
         //safe_authenticate ();
         // Set the basic data
         $data['page_title'] = 'Application Data Manager';
-        $data['page_summary'] = 'This page allows managing Application Data.';
+        $data['page_summary'] = 'This page allows managing application data.';
         $data['page_large_icon'] = '/images/icons/32x32/icon-backup-settings.png';
         $data['page_small_icon'] = '/images/icons/16x16/icon-backup-settings.png';
         //Prepare table structure
@@ -172,21 +172,15 @@ class Sng_data_manager extends Safe_CI_Controller
         if ($data_cnt && $data_cfg['global_capabilities']) $glob_caps = array_merge($glob_caps, $data_cfg['global_capabilities']);
         $global_opt = "";
         foreach ($glob_caps as $glob_cap) {
-            if($glob_cap == 'upload'){
-                $tmp_button = str_replace('type="submit"','type="button" onclick="safe_handle_create_button(this);"',safe_form_with_submit_button('', '/SAFe/' . $this->router->fetch_class() . '/' . $glob_cap .'_form'. '/' . $module_name . '/' . $data_name, ucfirst($glob_cap), $glob_cap, 'POST', 'float:left;'));
-                $global_opt.= '<div>'.$tmp_button.'</div>'; 
-            }elseif($glob_cap == 'backup'){
-                $tmp_button = str_replace('type="submit"','type="button" onclick="safe_handle_create_button(this);"',safe_form_with_submit_button('', '/SAFe/' . $this->router->fetch_class() . '/' . $glob_cap .'_form'. '/' . $module_name . '/' . $data_name, ucfirst($glob_cap), $glob_cap, 'POST', 'float:left;'));
-                $global_opt.= '<div>'.$tmp_button.'</div>';
+            if($glob_cap == 'upload' || $glob_cap == 'backup'){
+                $global_opt.= safe_form_action_button('', '/SAFe/' . $this->router->fetch_class() . '/' . $glob_cap .'_form'. '/' . $module_name . '/' . $data_name, ucfirst($glob_cap), $glob_cap, 'POST', 'float:left;');
             }else{
                 $tmp_button_array = array_pop(safe_format_button($glob_cap));
-                $global_opt.= '<div style="float:left">'.safe_button_click($tmp_button_array['id'], ucfirst($tmp_button_array['value']) , '/SAFe/' . $this->router->fetch_class() . '/' . $tmp_button_array['id'] . '/' . $module_name . '/' . $data_name, array(
-                        'float:right;'
-                )).'</div>';
+                $global_opt .= safe_button_click($tmp_button_array['id'], ucfirst($tmp_button_array['value']) , '/SAFe/' . $this->router->fetch_class() . '/' . $tmp_button_array['id'] . '/' . $module_name . '/' . $data_name, '');
             }
         }
         if ($global_opt) {
-            $data['notif_warn'][] = '<div style="float:left;height:22px;line-height:22px;">Global Options for ' . $data_cfg['description'] . ':&nbsp;&nbsp;</div>' . $global_opt;
+            $data['notif_warn'][] = safe_form_global_action('Global Options for ' . $data_cfg['description'], $global_opt);
         }
         // Prepare the list view with data
         $table = new Safe_crud_table_class();
@@ -292,7 +286,7 @@ class Sng_data_manager extends Safe_CI_Controller
         //safe_authenticate ();
         // Set the basic data
         $data['page_title'] = 'Application Data Manager';
-        $data['page_summary'] = 'This page allows managing Application Data.';
+        $data['page_summary'] = 'This page allows managing application data.';
         $data['page_large_icon'] = '/images/icons/32x32/icon-backup-settings.png';
         $data['page_small_icon'] = '/images/icons/16x16/icon-backup-settings.png';
         $module = NULL;
@@ -470,7 +464,7 @@ class Sng_data_manager extends Safe_CI_Controller
         $output = array();
         if($module->upload_data($data_name, NULL, $output, 'POST')){
             $data['message_type'] = 'notice';
-            $data['message'] ='File Uploaded Successfully.';
+            $data['message'] ='File uploaded successfully.';
             $data['result'] = true;
             $data['redirect_url'] = $back_url;
         }else{
