@@ -90,7 +90,7 @@ class Resolver extends Network
 	function ModificationAllowed()
 	{
 		/* hack copied from network.php */
-		exec("pidof XXX_SNG_PRODUCT_NAME_SHORT_XXX", $output);
+		exec("pidof nsc", $output);
 		return ($output[0] ==  0);
 	}
 
