@@ -47,6 +47,7 @@ require_once(COMMON_CORE_DIR . '/api/Webconfig.class.php');
 ///////////////////////////////////////////////////////////////////////////////
 
 // TODO: extra XSS protection
+/*
 function WebCheckFormVariables($item, $key)
 {
 	// TODO: bad hack to allow <> in passwords.  Will be going away in 6.0.
@@ -58,7 +59,7 @@ function WebCheckFormVariables($item, $key)
 }
 
 array_walk_recursive($_REQUEST, 'WebCheckFormVariables');
-
+ */
 ///////////////////////////////////////////////////////////////////////////////
 // C O N S T A N T S
 ///////////////////////////////////////////////////////////////////////////////
@@ -1029,26 +1030,13 @@ function WebUrlJump($url, $description)
 
 function WebAuthenticate()
 {
-	// Forward to wizard when required
-	//--------------------------------
-
-	if (file_exists(Webconfig::FILE_SETUP_FLAG) && 
-		!preg_match("/\/admin\/setup\..*/", $_SERVER['PHP_SELF']) &&
-		!(WEBCONFIG_CONSOLE)
-		) {
-		// TODO: not very clean... the wizard needs to pull in Ajax helper pages
-		if (!(preg_match("/\.js\./", $_SERVER['PHP_SELF']) || preg_match("/\.xml\./", $_SERVER['PHP_SELF']))) {
-			WebForwardPage("/admin/setup.php");
-			exit;
-		}
-	}
-
 	// Logout requested
 	//-----------------
 
 	if (isset($_REQUEST['reserved_logout']))  {
 		// We need the session data for formatting, so don't destroy the
 		// session before grabbing our "you have been logged out" HTML.
+    Logger::SysLog("webconfig", "logout - ".$_SESSION['user_login']);
 
     ob_start();
 		WebHeader($_SESSION['system_osname'], false);
@@ -1123,38 +1111,37 @@ function WebAuthenticate()
  * authentication is validated through /etc/passd.
 
 			if ($username == "root") {
-*/
-      if (! file_exists(COMMON_CORE_DIR . '/api/PosixUser.class.php'))
-        exit();
 
-      require_once(COMMON_CORE_DIR . '/api/PosixUser.class.php');
-
-      try {
-        $user = new PosixUser($username);
-        $passwordok = $user->CheckPassword($password);
-      } catch (Exception $e) {
-        WebHeader("", false);
-        WebDialogWarning($e->GetMessage());
-        Webfooter();
-        exit();
-      }
-/*		} else {
-        if (! file_exists(COMMON_CORE_DIR . '/api/User.class.php'))
+        if (! file_exists(COMMON_CORE_DIR . '/api/PosixUser.class.php'))
           exit();
 
-        require_once(COMMON_CORE_DIR . '/api/User.class.php');
+        require_once(COMMON_CORE_DIR . '/api/PosixUser.class.php');
 
         try {
-          $user = new User($username);
-          $passwordok = $user->CheckPassword($password, 'pcnWebconfigPassword');
-      } catch (Exception $e) {
-        WebHeader("", false);
-        WebDialogWarning($e->GetMessage());
-        Webfooter();
-        exit();
+          $user = new PosixUser($username);
+          $passwordok = $user->CheckPassword($password);
+        } catch (Exception $e) {
+          WebHeader("", false);
+          WebDialogWarning($e->GetMessage());
+          Webfooter();
+          exit();
+        }
+      } else {
+          if (! file_exists(COMMON_CORE_DIR . '/api/User.class.php'))
+            exit();
+
+          require_once(COMMON_CORE_DIR . '/api/User.class.php');
+
+          try {
+            $user = new User($username);
+            $passwordok = $user->CheckPassword($password, 'pcnWebconfigPassword');
+        } catch (Exception $e) {
+          WebHeader("", false);
+          WebDialogWarning($e->GetMessage());
+          Webfooter();
+          exit();
+        }
       }
-    }
- */
 			if (($username == "root") && $passwordok) {
 				Logger::SysLog("webconfig", "login - root login successful");
 				$_SESSION['system_login'] = "root";
@@ -1182,7 +1169,37 @@ function WebAuthenticate()
 				exit;
 			}
 		}
-	}
+ */
+      // Check user login using /usr/local/sng/bin/sng-user-mgmt
+      require_once(COMMON_CORE_DIR . '/api/ShellExec.class.php');
+      $shell = new ShellExec();
+      $cmd = '/usr/local/sng/bin/sng-user-mgmt';
+      $args =' --action=login --user='.$username.' --password='.$password;
+      $rc = $shell->Execute($cmd, $args, true, array('log'));
+      if(0 == $rc){
+        Logger::SysLog("webconfig", "login - ".$username." login successful");
+        $args_info = ' --user='.$username.' --action=info';
+        $rc2 = $shell->Execute($cmd, $args_info, true, array('log'));
+        $output = $shell->GetFirstOutputLine();
+        if($output){
+            $output_obj = json_decode(str_replace("'",'"',$output));
+            $_SESSION['user_name'] = $output_obj->info;
+        }else{
+            $_SESSION['user_name'] = $username;
+        }
+        $_SESSION['user_login'] = $username;
+        $_SESSION['system_login'] = "root";
+        WebSetSessionAuthenticated();
+      }else{
+        Logger::SysLog("webconfig", "login - ${username} login failed");
+
+        WebHeader($_SESSION['system_osname'], false);
+        WebAuthenticateDisplayLogin($username, $password, WEBCONFIG_LANG_ERRMSG_LOGIN_FAILED);
+        WebFooter(false);
+        exit;
+      }
+    }
+  }
 }
 
 /**
@@ -1399,7 +1416,7 @@ function WebSetSession()
 
 	// Hostkey
 	//--------
-
+/*
 	$hostkey = "hostkey";
 
 	if (file_exists(COMMON_CORE_DIR . "/api/Suva.class.php")) {
@@ -1412,7 +1429,7 @@ function WebSetSession()
 			// Use default
 		}
 	}
-
+ */
 	// Template
 	//---------
 
